/*
 * TanhNeuron.java
 *
 * Created on August 10, 2000, 3:15 PM
 */
 
package saaps.ann.mlff;

import saaps.ann.Data;
import saaps.ann.Weights;
import java.io.Serializable;

/** 
 * Implementation of the tanh neuron. The output from the neuron
 * lies in the range [-1,+1].
 *
 * @author  Peter Wintoft
 * @version 1.0 2000-08-11
 */
public class TanhNeuron extends Neuron implements Serializable {

  /** Creates new TanhNeuron */
  public TanhNeuron() {
  }
  
  /**
  Create a tanh neuron from a set of weights and bias.
  
  @param  w The weights and bias
  */
  public TanhNeuron(Weights w)
  {
    super(w);
  }
  
  /**
  The tanh transfer function.
  
  @param  x The input to the transfer function
  @return The output from the transfer function
  */
  public Data transFun(Data x)
  {
    Data d = new Data(x);
    for (int c=0;c<d.getNumOfConnections();c++)
    {
      for (int e=0;e<d.getNumOfExamples();e++)
      {
        double val = d.get(c,e);
        d.set(c,e,tanh(val));
      }
    }
    return d;
  }
  
  /**
  Definition of the exponential function.
  
  @param  x Input to the function.
  @return The output from the function.
  */
  private double exp(double x)
  {
    return Math.exp(x);
  }
  
  /**
  Definition of the tanh function.
    
  @param  x Input to the function.
  @return The output from the function.
  */
  private double tanh(double x)
  {
    return (1.0 - exp(-2.0*x))/(1.0 + exp(-2.0*x));
  }

  
  
}