/*
 * BuildDst.java
 *
 * Created on March 1, 2001, 10:15 AM
 */
 
package saaps.dbt;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;
import java.util.Vector;

import saaps.util.IllegalTimeSeriesException;
import saaps.util.ObjectTimeSeries;

/** 
 * This class builds the Dst database from NGDC. The index is the final Dst.
 *
 * @author  Peter Wintoft
 * @version 1.0
 */
public class BuildDst extends Object {

  /** Creates new BuildDst */
  public BuildDst() {
  }
  
  /**
  Connects to the ftp server, goes through the list of available data, downloads
  the data, and stores it into the SAAPS database.
  <p>
  Currently (2001-03-01) final Dst data exist for 1957 to 1998.
  */
  public void run() throws Exception {
    //First year and last year to read.
    int yearFirst = 1957;
    int yearLast = 2000;
    //Create the DstData object.
    DstData dstData = new DstData();
    //Go through the years.
    for (int year=yearFirst;year<=yearLast;year++) {
      System.out.println("Reading year " + year + " ...");
      dstData.setYear(year);
      //Create vector to hold one year of data.
      Vector vec = new Vector(24*366);
      //Go through for all days for current year.
      while (dstData.readLine()) {
        //Go through the 24 hours of each day.
        for (int hour=0;hour<24;hour++) {
          dstData.setHour(hour);
          //Create Dst object.
//          System.out.println(dstData.getDate());            
          Dst dst = new Dst(dstData.getDate(),dstData.getData());
          //Add to vector of objects.
          vec.addElement(dst);
        }
      }
      //Trim the vector.
      vec.trimToSize();
      System.out.println("Finished reading year " + year + "!");
      try {
        dstData.close();
      }
      catch (IOException ioe) {
        System.err.println(ioe);
        System.exit(0);
      }
      //Store in database.
      System.out.print("Writing to database ... ");
      //Store in database if the vector contain data.
      if (!vec.isEmpty()) {
        try {
          ObjectTimeSeries ts = new ObjectTimeSeries(vec);
          Database db = Database.getInstance("DstDB");
          db.store(ts);
          System.out.println("Done!");
        }
        catch (IllegalTimeSeriesException e) {
          System.err.println(e);
        }
      }
      else {
        System.out.println("Failed! No data to store.");
      }
    }
  }
  
  class DstData {
    BufferedReader bufReader;
    String textLine = "Nonsence";
    Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
    int hour = 0;
    
    public DstData() {
      //Clear the fields in the calendar object.
      cal.clear();
    }
    
    public void setYear(int year) {
      //The ftp address.
//      String urlString = "ftp://ftp.ngdc.noaa.gov/STP/GEOMAGNETIC_DATA/INDICES/DST/dst" + year;
      //Connect and get the list.
      try {
//        URL url = new URL(urlString);
//        System.out.println(url.toString());
//        InputStream inStream = url.openStream();
//        bufReader = new BufferedReader(new InputStreamReader(inStream));
        bufReader = new BufferedReader(new FileReader("DatabaseSrc/Dst/dst" + year));
      }
//      catch (MalformedURLException e) {
//        System.err.println(e);
//        System.exit(0);
//      }
      catch (IOException e) {
        System.err.println(e);
        System.exit(0);
      }
    }
    
    public void close() throws IOException {
//      System.out.println("DstData.close()");
      this.bufReader.close();
//      System.out.println("DstData.close() finished");
    }
    
    public boolean readLine() {
//      System.out.println("DstData.readLine(): start ...");
      try {
        this.textLine = this.bufReader.readLine();
      }
      catch (IOException e) {
        System.err.println(e);
      }
      return (this.textLine != null);
//      System.out.println("DstData.readLine(): finished!");
//      System.out.println(this.textLine);
    }
    
    public void setHour(int hour) {
      this.hour = hour;
    }
    
    public Date getDate() {
      try {
        //Get the year.
        //If the two digit year (yy) is less than 57 assume that the year is 2000 + yy,
        //else 1900 + yy.
        int yy = Integer.parseInt(this.textLine.substring(3,5));
        int year;
        if (yy<57) {
          year = 2000 + yy;
        }
        else {
          year = 1900 + yy;
        }
        //Get the month.
        int month = Integer.parseInt(this.textLine.substring(5,7));
        //Get the day.
        int day = Integer.parseInt(this.textLine.substring(8,10));
        //Set the date.
        this.cal.set(Calendar.YEAR,year);
        this.cal.set(Calendar.MONTH,month-1);
        this.cal.set(Calendar.DATE,day);
        this.cal.set(Calendar.HOUR_OF_DAY,this.hour);
      }
      catch (NumberFormatException e) {
        System.err.println(e);
      }
      return cal.getTime();
    }
    
    public double getData() {
      double data = 0;
      int start = 20 + 4*this.hour;
      int end = 24 + 4*this.hour;
      try {
        data = Double.parseDouble(this.textLine.substring(start,end));
      }
      catch (NumberFormatException e) {
        System.err.println(e);
      }
      return data;
    }
  }
  
/*  class DstList {
    BufferedReader bufReader;
    public DstList() {
      //The ftp address.
      String urlString = "ftp://ftp.ngdc.noaa.gov/STP/GEOMAGNETIC_DATA/INDICES/DST/";
      //Connect and get the list.
      try {
        URL url = new URL(urlString);
        InputStream inStream = url.openStream();
        InputStreamReader inStreamReader = new InputStreamReader(inStream);
        bufReader = new BufferedReader(inStreamReader);
        try {
          while (true) {
            System.out.println(bufReader.readLine());
          }
        }
        catch (IOException e) {
          System.out.println(e);
          System.out.println("Reached end of file.");
        }
      }
      catch (MalformedURLException e) {
        System.err.println(e);
      }
      catch (IOException e) {
        System.err.println(e);
      }
    }
  }*/
  
  /**
  * @param args the command line arguments
  */
  public static void main (String args[]) {
    //Create the build Dst class.
    BuildDst dst = new BuildDst();
    //Run the program.
    try {
        dst.run();
    }
    catch (Exception e) {
        System.err.println(e);
    }
  }

}