package saaps.dbt;

import java.io.*;
import java.util.*;
import java.text.*;
import java.lang.*;

import saaps.dbt.ODIDatabase;
import saaps.util.TimeSeries;

/**
 * Handles the export of data from the SAAPS database. It can export
 * data from all Database classes.
 * The exported data has the following structure:
 * <p>
 * YYYY MM DD HH MM SS VAL(1) VAL(2) ... VAL(N)
 * <p>
 * See also {@link saaps.dbt.ImportData}.
 * @author Leif Kalla (leif@irf.se)
 */
public class ExportAscii  {

  //An object to the ODI database.
  private ODIDatabase odiDatabase;

//  Database dbRef;
  String useThis;

  public ExportAscii(String cname) {
    
    useThis = cname;
    this.odiDatabase = new ODIDatabase();

    /* Get database object and open raf for read only */
//    dbRef = Database.getInstance(useThis,true);
  }
  
  public void export(Date startDate, Date endDate, String fileName) throws IOException {
	BufferedWriter writer = new BufferedWriter(new FileWriter(fileName));
	TimeSeries ts = odiDatabase.retrieveTS(useThis, startDate, endDate, false);

	Calendar sd = Calendar.getInstance(TimeZone.getTimeZone("UTC")
					   ,new Locale("sv","SE"));
	DateFormat mf = DateFormat.getDateTimeInstance();
	mf.setCalendar(sd); //Set our locale and UTC
	SimpleDateFormat formatter = (SimpleDateFormat)
	SimpleDateFormat.getDateTimeInstance(0,0,new Locale("sv","SE"));

	formatter.setTimeZone(TimeZone.getTimeZone("UTC"));
	formatter.applyPattern("yyyy MM dd HH mm ss");

	  for(int i=0;i<ts.getLength();i++) {
	    double[] row = ts.getData(i);
	    Vector rowv = new Vector(2000);
            writer.write(formatter.format(ts.getDate(i)));
	    for(int j=0;j<ts.getNumOfDataFields();j++) {
		rowv.add(row[j]);
		writer.write(" " + (rowv.elementAt(j)).toString());
	    }
	    writer.write("\n");
//	    System.out.println((dboVec.elementAt(i)).toString());
	}

//writer.write("\nfn:"+fileName);
//writer.write("\nnumberofdata:"+ts.getLength());
//writer.write("\nnumberofdatafields:"+ts.getNumOfDataFields());
	writer.close();
      
  }
/*  public void export(Date startDate, Date endDate, String fileName) throws IOException {
      BufferedWriter writer = new BufferedWriter(new FileWriter(fileName));
      if(this.dbRef.isSet()) {
          
          int MAXSIZE=209715200; //200Mb ,1048576; // bytes , 1Mb of memory buffer
          long timeInterval = 60*60*24*1000; //Default is 1 day

          try {
                // If the db is continous we will figure out how many objects 
                //
	        // will fit in MAXSIZE of ram
	        //
	        if(this.dbRef.meta.getCont()) {
	            long objSize = this.dbRef.getFirst().getObjectSize();
	            int maxObjs = (int) (MAXSIZE/objSize);
	            //estimate time interval
	            timeInterval = maxObjs * this.dbRef.meta.getTimeStep();
	        }

	        Vector dboVec = new Vector(2000);
	        Object dbo=null;
    
//	        String dateTime = new String(args[1] + " " + args[2]);

        	Calendar ed = Calendar.getInstance(TimeZone.getTimeZone("UTC"),
					   new Locale("sv","SE"));

        	Calendar tmped = Calendar.getInstance(TimeZone.getTimeZone("UTC"),
					      new Locale("sv","SE"));

        	Calendar tmpsd = Calendar.getInstance(TimeZone.getTimeZone("UTC")
					      ,new Locale("sv","SE"));
        	DateFormat mf = DateFormat.getDateTimeInstance();
        	mf.setCalendar(sd); //Set our locale and UTC

//	SimpleDateFormat formatter = (SimpleDateFormat)
//	  SimpleDateFormat.getDateTimeInstance(0,0,new Locale("sv","SE"));

//	formatter.setTimeZone(TimeZone.getTimeZone("UTC"));
//	formatter.applyPattern("yyyy.MM.dd HH:mm:ss");
     
	// Parse the string back into a Date.
//	ParsePosition pos = new ParsePosition(0);
//	Date inputDateTime = formatter.parse(dateTime, pos);

	// Our selected start time
//	sd.setTime(inputDateTime);
	sd.setTime(startDate);
	sd.clear(sd.MILLISECOND);
	// our selected end time
//	ed.setTime(new Date ((long) sd.getTime().getTime() +
//			     (new Long(args[3])).longValue()*60*1000));
        ed.setTime(endDate);
        ed.clear(ed.MILLISECOND);

	if(sd.before(this.dbRef.meta.getStartCalendar()))
            sd.setTime(this.dbRef.meta.getStartCalendar().getTime());
	//Start buffer 
	tmpsd.setTime(sd.getTime());
	tmpsd.clear(sd.MILLISECOND);
	//end buffer
	tmped.setTime(new Date( (long) sd.getTime().getTime()+timeInterval));
//	tmped.setTime(ed.getTime());
	tmped.clear(sd.MILLISECOND);

	//Checks
	if(ed.before(tmpsd)) {
	  System.out.println("No data");
	  System.exit(0);
	}
	if(ed.before(tmped))
	  tmped.setTime(ed.getTime());

	while(tmped.before(ed) || tmped.equals(ed)) {
	  dboVec=this.dbRef.retrieve(tmpsd.getTime(),tmped.getTime(),false);
	  for(int i=0;i<dboVec.size();i++) {
              writer.write((dboVec.elementAt(i)).toString() + "\n");
//	    System.out.println((dboVec.elementAt(i)).toString());
	  }
	  tmpsd.setTime(tmped.getTime());
	  tmpsd.add(tmpsd.MILLISECOND,1); //Add one millisec so that we get next
	  tmped.setTime(new Date( (long) tmpsd.getTime().getTime()+timeInterval));	  
	}

	if(tmpsd.before(ed)) { //The remaining part
	  dboVec=this.dbRef.retrieve(tmpsd.getTime(),ed.getTime(),false);
	  for(int i=0;i<dboVec.size();i++) {
              writer.write((dboVec.elementAt(i)).toString() + "\n");
//	    System.out.println((dboVec.elementAt(i)).toString());
	  }
	}
      }
      catch (IOException e) {
	System.err.println(e);
	e.printStackTrace();
      }
    }
    else
      System.out.println("No meta data available");
    try {
      this.dbRef.close();
    }
    catch (IOException ee) {
      System.err.println(ee);
      ee.printStackTrace();
    }
      writer.close();
  }

  static public void main(String[] args) {

    if(args.length<4) {
      System.out.println("Usage: java saaps.dbt.ExportAscii <Db class name> <yyyy.mm.dd> <hh:mm:ss> <duration> \n   duration in minutes");
      System.exit(0);
    }

    ExportAscii dl = new ExportAscii(args[0]);
    if(dl.dbRef.isSet()) {

      final int MAXSIZE=209715200; //200Mb ,1048576; // bytes , 1Mb of memory buffer
      long timeInterval = 60*60*24*1000; //Default is 1 day

      try {
 
	// If the db is continous we will figure out how many objects 
	// will fit in MAXSIZE of ram
	//
	if(dl.dbRef.meta.getCont()) {
	  long objSize = dl.dbRef.getFirst().getObjectSize();
	  int maxObjs = (int) (MAXSIZE/objSize);
	  //estimate time interval
	  timeInterval = maxObjs * dl.dbRef.meta.getTimeStep();
	}

	Vector dboVec = new Vector(2000);
	Object dbo=null;
    
	String dateTime = new String(args[1] + " " + args[2]);

	Calendar sd = Calendar.getInstance(TimeZone.getTimeZone("UTC")
					   ,new Locale("sv","SE"));
	Calendar ed = Calendar.getInstance(TimeZone.getTimeZone("UTC"),
					   new Locale("sv","SE"));

	Calendar tmped = Calendar.getInstance(TimeZone.getTimeZone("UTC"),
					      new Locale("sv","SE"));

	Calendar tmpsd = Calendar.getInstance(TimeZone.getTimeZone("UTC")
					      ,new Locale("sv","SE"));
	DateFormat mf = DateFormat.getDateTimeInstance();
	mf.setCalendar(sd); //Set our locale and UTC

	SimpleDateFormat formatter = (SimpleDateFormat)
	  SimpleDateFormat.getDateTimeInstance(0,0,new Locale("sv","SE"));

	formatter.setTimeZone(TimeZone.getTimeZone("UTC"));
	formatter.applyPattern("yyyy.MM.dd HH:mm:ss");
     
	// Parse the string back into a Date.
	ParsePosition pos = new ParsePosition(0);
	Date inputDateTime = formatter.parse(dateTime, pos);

	// Our selected start time
	sd.setTime(inputDateTime);
	sd.clear(sd.MILLISECOND);
	// our selected end time
	ed.setTime(new Date ((long) sd.getTime().getTime() +
			     (new Long(args[3])).longValue()*60*1000));
	ed.clear(ed.MILLISECOND);

	if(sd.before(dl.dbRef.meta.getStartCalendar()))
	  sd.setTime(dl.dbRef.meta.getStartCalendar().getTime());
	//Start buffer 
	tmpsd.setTime(sd.getTime());
	tmpsd.clear(sd.MILLISECOND);
	//end buffer
	tmped.setTime(new Date( (long) sd.getTime().getTime()+timeInterval));
	tmped.clear(sd.MILLISECOND);

	//Checks
	if(ed.before(tmpsd)) {
	  System.out.println("No data");
	  System.exit(0);
	}
	if(ed.before(tmped))
	  tmped.setTime(ed.getTime());

	while(tmped.before(ed) || tmped.equals(ed)) {
	  dboVec=dl.dbRef.retrieve(tmpsd.getTime(),tmped.getTime(),false);
	  for(int i=0;i<dboVec.size();i++) {
	    System.out.println((dboVec.elementAt(i)).toString());
	  }
	  tmpsd.setTime(tmped.getTime());
	  tmpsd.add(tmpsd.MILLISECOND,1); //Add one millisec so that we get next
	  tmped.setTime(new Date( (long) tmpsd.getTime().getTime()+timeInterval));	  
	}

	if(tmpsd.before(ed)) { //The remaining part
	  dboVec=dl.dbRef.retrieve(tmpsd.getTime(),ed.getTime(),false);
	  for(int i=0;i<dboVec.size();i++) {
	    System.out.println((dboVec.elementAt(i)).toString());
	  }
	}
      }
      catch (IOException e) {
	System.err.println(e);
	e.printStackTrace();
      }
    }
    else
      System.out.println("No meta data available");
    try {
      dl.dbRef.close();
    }
    catch (IOException ee) {
      System.err.println(ee);
      ee.printStackTrace();
    }
  }*/
}
