package saaps.dbt;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;

import saaps.util.*;

/**
* ImportKp reads Kp from file and merges it into a database.
*
*
*@author Leif Kalla
*@version 0.1 2000-02-07
*/
public class ImportKp {
  private static double dataGapValue = -1.0;

  public static void main(String[] args) {
    Date inputDateTime;
    String tmpVal;
    double tmpKp;

    if(args.length > 0) {
      System.out.println("Usage: java saaps.saam.ImportKp <stdinput>");
      System.exit(0);
    }

    try {
      // Input stream.
      BufferedReader streamIn = new BufferedReader(new InputStreamReader(System.in));
      String textLine = "";
      textLine = streamIn.readLine();
      // Go through the file for all lines starting with
      // "#", ":", or " ".
      while (!textLine.startsWith("1") && !textLine.startsWith("2")) {
	textLine = streamIn.readLine();
      }
      // Create a Vector of Kp objects.
      Vector kpv = new Vector(8*30);
      do {
	// Read the date using - as the field separator.
	String dateStr = textLine.substring(0,4) + "." +
	  textLine.substring(5,7) + "." +
	  textLine.substring(8,10) + " " + textLine.substring(11,13) + ":" + textLine.substring(14,16) + ":" + textLine.substring(17); //Last item to end of string

	SimpleDateFormat formatter = (SimpleDateFormat) SimpleDateFormat.getDateTimeInstance(0,0,new Locale("sv","SE"));

	formatter.setTimeZone(TimeZone.getTimeZone("UTC"));
	formatter.applyPattern("yyyy.MM.dd HH:mm:ss");
     
	// Parse the string back into a Date.
	ParsePosition pos = new ParsePosition(0);
	inputDateTime = formatter.parse(dateStr, pos);
	
	// Assume that observations are at 00, 03, 06, ...
	Calendar cal = Calendar.getInstance();
	cal.setTimeZone(formatter.getTimeZone());
	cal.setTime(inputDateTime);
	//The data value
	tmpVal = textLine.substring(20).trim();
	if(tmpVal.equals(new String("NaN"))) {
	  tmpKp = Double.NaN;
	}
	else {
	   tmpKp = Double.valueOf(tmpVal).doubleValue();
	}
	if (tmpKp == dataGapValue ) {
	  tmpKp = Double.NaN;
	}

	//System.out.println("Date/tome:" + formatter.format(inputDateTime) + " " + tmpKp);

	kpv.addElement(new Kp(cal.getTime().getTime(),tmpKp));
	
      } while ((textLine = streamIn.readLine()) != null);

      // Trim the vector.
      kpv.trimToSize();
      // Close input stream.
      streamIn.close();
      // Store the data in the database
      //Database db = Database.getInstance("KpDB",kpv);
      //OK let us make a TimeSeries :)
      ObjectTimeSeries ts = new ObjectTimeSeries(kpv);
      //Check if TimeSeries is OK to insert into DB
      //ts.checkDbValid(new MetaData("KpDB")); is done in the Database Store method

      Database db = Database.getInstance("KpDB");

      if(!db.isSet())
	System.out.println("MetaData for Kp is unavailable \nWill try to create it.");

      db.store(ts);
    }
    catch (IOException e) {
      System.err.println("IO error: " + e);
    }
    catch (NumberFormatException e) {
      System.err.println("Number format error: " + e);
    }
    catch (IllegalTimeSeriesException e) {
      System.err.println(e);
    }
    catch (Exception e) {
      System.err.println("IO error: " + e);
    }

//     catch (TimeSeriesSequenceException e) {
//       System.err.println(e);
//       //e.printStackTrace();
//     }

//     catch (TimeSeriesContinuityException e) {
//       System.err.println(e);
//       //e.printStackTrace();
//     }

//     catch (IllegalTimeSeriesException e) {
//       System.err.println(e);
//       //e.printStackTrace();
//     }
  }
}
