package saaps.dbt;

import java.io.*;
import java.util.*;

/**
 * LANL1990_095DB extends the Database class. It contains
 * the read and write methods that are used in
 * Database. The field is a vector of LANL1990_095 objects.
 *
 *@author Peter Wintoft
 *@version 1.0 2001-02-13
 *@see LANL1990_095
 */
public class LANL1990_095DB extends Database {
  // Absolute path to the database.
  //  private String baseDir = "/space/saaps/Database/";

  private static String cName = "LANL1990_095";

  private static final double dataGapValueDefinition = -999.0;
  private static double[] dataGapValues =
  {-999.0, -999.0, -999.0, -999.0, -999.0, -999.0, -999.0, -999.0, -999.0, -999.0};

  // Static Fields used to initiate the MetaData
  private static String name= cName + "DB";
  private static String description = "LANL 1990-095 data";
  private static String longDescription = "The LANL 1990-095 data set contain several different " +
    "parameters with a one hour resolution. This data set is only a subset of the full LANL set. " +
    "This data set contain the geosynchronous electron flux for the following energy levels:" + 
    "21.6-28.9 eV, 87-114 eV, 0.793-1.04 keV, 9.23-12.14 keV, 75-105 keV, 0.75-1.1 MeV, and 7.8-10.8 MeV." +
    "The latitude, longitude, and radius are also included.";
  private static Calendar st=Calendar.getInstance(TimeZone.getTimeZone("UTC"),new Locale("sv","SE"));
  private static Calendar et=Calendar.getInstance(TimeZone.getTimeZone("UTC"),new Locale("sv","SE"));
  private static boolean ct=true;
  private static long step=60*60*1000; //One hour timestep
  private static String[] fieldNames =
    { "Latitude", "Longitude", "Radius", "E21.6-28.9 eV", "E87-114 eV", "E0.793-1.04 keV",
      "E9.23-12.14 keV", "E75-105 keV", "E0.75-1.1 MeV", "E7.8-10.8 MeV"};
  private static String[] fieldUnits =
    { "degrees north", "degrees east", "Re",
      "1/(cm^2 s sr)", "1/(cm^2 s sr)", "1/(cm^2 s sr)", "1/(cm^2 s sr)",
      "1/(cm^2 s sr)", "1/(cm^2 s sr)", "1/(cm^2 s sr)"};
  private static String[] fieldTypes =
    { "Double", "Double", "Double",
      "Double", "Double", "Double", "Double", "Double","Double", "Double"};
  private static int numOfFields = fieldNames.length;

  // Fields.
  private LANL1990_095 data;
  private Vector vec;

  // Constructors.
  public LANL1990_095DB() {
    
  }
  public LANL1990_095DB(int i) {
    
    //    acev = ACEV;
    //store(acev);
  }
  /**
   * When a new class is added to the database this
   * constructor should be used which includes the
   * information that goes into the meta data file.
   *
   * The arguments are: Vector of LANL1990_095 objects, the class name,
   * a short description, continuos time series (false/true), and
   * if a continous time series the time step.
   */
  public LANL1990_095DB(Vector VEC) throws IOException {
    //, String des, boolean ct, long step) {
    //
    
    vec = VEC;
    raf = new RandomAccessFile(baseDir + cName + "DB.data","rw");
    // Then get its metadata
    try { // To load the meta data for this database from file
      meta = new MetaData(cName + "DB");
      isSet=true;
    }
    catch (IOException e) {
      System.err.println(e);
      System.out.println("Here we should handle the situation when no DB exists");
      try {
  meta = setDefaultMeta(((DatabaseObject) (vec.elementAt(0))).getDate(),((DatabaseObject) (vec.elementAt(vec.size()-1))).getDate()); //Takes static fields from XDB

  isSet=true;
      }
      catch (IOException ee) {
  System.err.println(ee);
  ee.printStackTrace();
      }
    }
    catch (ClassNotFoundException e) {
      System.err.println(e);
      e.printStackTrace();
    }

    store((Vector) vec); //Need to type cast so that store(Object obj) isn't used
  }

  // Methods.
  public Object read() throws IOException {
    long t = raf.readLong();
    double[] d = new double[numOfFields];
    for (int i=0;i<numOfFields;i++)
      d[i] = raf.readDouble();
    data = new LANL1990_095(t,d);
    return (Object) data;
  }

  public void myHello() {
    System.out.println("My" + cName + "DB Hello");
  }

  public void write(Object obj) throws IOException {
    data = (LANL1990_095) obj;
    raf.writeLong(data.getDate().getTime());
    for (int i=0;i<numOfFields;i++)
      raf.writeDouble(data.getData(i));
  }
  // Implemented for Database abstract getDate
  public Date getDate(Object obj) {
    return ((LANL1990_095) obj).getDate();
  }

  /** This method is implemented for Database abstract metod.
   * It retrieves the latest data from the dataprovider and returns the vector of Database objects.
   */
  public Vector getLatestDatabaseObjects() {
    Vector vec = new Vector(24);
    Date mdate = new Date();
    double[] x = new double[numOfFields];
    for (int i=0;i<numOfFields;i++)
      x[i] = (double) i+1;

    LANL1990_095 lanl = new LANL1990_095(mdate,x);

    for(int i=0;i<8*30;i++) {
      vec.addElement(lanl);
      mdate.setDate( (int) (mdate.getDate() + 10));
      lanl.setDate(mdate);
      lanl.setData(x);
    }
    vec.trimToSize();
    return vec;
  }

  /** Creates a metadata file for this class with the default values set
   *to the static values defined above. OBS!, check calendar ....
   */
  protected MetaData setDefaultMeta(Date sd, Date ed) throws IOException {

    st.setTime(sd);
    st.clear(st.MILLISECOND);
    et.setTime(ed);
    et.clear(et.MILLISECOND);

    MetaData m = new MetaData(name,description, longDescription, st, et, ct, step,fieldNames,fieldUnits,fieldTypes);
    return m;
  }

  /**
   * Returns an "empty" metadata object. All fields but Start and end time
   * has valid values (as set in the respective database class).
   * The times will be set to that what is supplied.
   * OBS! This is not stored in the MetaData file !!
   */
  protected MetaData getEmptyMetaData(Date sd, Date ed) {
    MetaData m = new MetaData();
    st.clear(st.MILLISECOND);
    st.setTime(sd);
    et.clear(et.MILLISECOND);
    et.setTime(ed);
    m.setMetaData(name,description, longDescription, st, et, ct, step, fieldNames, fieldUnits, fieldTypes );
    return m;
  }

}
