package saaps.dbt;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;

import saaps.util.*;

/**
* LatestACEswe downloads the latest ACE plasma data from SEC
* and stores it in the database.
* FTP address: ftp://ftp.sel.noaa.gov/pub/lists/ace/ace_mag_1m.txt and
*	ftp://ftp.sel.noaa.gov/pub/lists/ace/ace_swepam_1m.txt
*
*@author Peter Wintoft
*@version 2.1 1999-12-21 (Change data gaps to NaN)
*@version 2.0 1999-09-22
*@version 1.0 990913
*/
public class LatestACEswe
{
  public static void main(String[] args)
    {
		String ftpSrcStr="ftp://ftp.sel.noaa.gov/pub/lists/ace/ace_swepam_1m.txt";
		DateFormat df;
		Date d;
		Vector acev = null;
		try
		{
			// Open connection
//			System.out.println("Open URL ...");
			URL url = new URL(ftpSrcStr);
			URLConnection urlConn = url.openConnection();
			// Input stream.
			BufferedReader streamIn = new BufferedReader(
			                            new InputStreamReader(
			                              urlConn.getInputStream()));
			String textLine = "";
			textLine = streamIn.readLine();
			// Go through the file for all lines starting with
			// "#", ":".
//			while (!textLine.startsWith("1") && !textLine.startsWith("2"))
			while (textLine.charAt(0)=='#' || textLine.charAt(0)==':')
			{
//				System.out.println("Skipping: " + textLine);
				textLine = streamIn.readLine();
			}
			// Create a Vector of ACE velocity and density objects.
			acev = new Vector(120);
			while ((textLine = streamIn.readLine()) != null)
			{
//				System.out.println("Reading: " + textLine);
				// Tokenize the string.
				StringTokenizer st = new StringTokenizer(textLine);
				// Create the date string.
				String dateStr = st.nextToken() + "-"	+		//Year
													st.nextToken() + "-" +	//Month
													st.nextToken() + " ";		//Day
				String timeStr = st.nextToken();					//Hour and minute
				// Insert a ":" between hour and minute.
				timeStr = timeStr.substring(0,2) + ":" + timeStr.substring(2,4);
				dateStr = dateStr + timeStr;
				// Read the date using - as the field separator.
//				String dateStr = textLine.substring(0,4) + "-" +
//				               		textLine.substring(5,7) + "-" +
//				               		textLine.substring(8,10) + " " +
//				               		textLine.substring(11,13) + ":" +
//				               		textLine.substring(13,15);
				// Use UTC time and get the Date object.
				df = DateFormat.getDateTimeInstance(DateFormat.SHORT,DateFormat.SHORT,new Locale("sv","SE"));
				df.setTimeZone(TimeZone.getTimeZone("UTC"));
				d = df.parse(dateStr);
				// Skip over until n field.
				st.nextToken();
				st.nextToken();
				st.nextToken();
				// Read n and V.
				double n = Double.valueOf(st.nextToken()).doubleValue();
				double v = Double.valueOf(st.nextToken()).doubleValue();
				// Check if data gaps.
				if (n==-9999.9)
				  n=Double.NaN;
				if (v==-9999.9)
				  v=Double.NaN;
				// Add data to vectors.
				acev.addElement( new ACEswe(d.getTime(),n,v));

				//aceVel.addElement(new Observation(d.getTime(),v));
				//aceDen.addElement(new Observation(d.getTime(),n));
			}
			acev.trimToSize();

			// Close input stream.
			streamIn.close();
//			System.out.println("Ready!");
			// Store the data in the database
			//Database db = Database.getInstance("ACEsweDB");
			//Database db = Database.getInstance("ACEsweDB",acev);

			ObjectTimeSeries ts = new ObjectTimeSeries(acev);

			Database db = Database.getInstance("ACEsweDB");

			if(!db.isSet())
			  System.out.println("MetaData for ACEsweDB is unavailable \nWill try to create it.");
			
			System.out.println("Got instance of DB will do store");
			db.store(ts);

			//db.createDatabase(acev);
			//db.close();

			//DatabaseSP dbVel = new DatabaseSP("ACE-V");
			//DatabaseSP dbDen = new DatabaseSP("ACE-N");
			//dbVel.store(aceVel);
			//dbDen.store(aceDen);
			//dbVel.close();
			//dbDen.close();
		}
		catch (IOException e)
		{
			System.err.println("URL Connection error: " + e);
		}
		catch (NumberFormatException e)
		{
			System.err.println("Number format error: " + e);
		}
		catch (ParseException e)
		{
			System.err.println("Parse error: " + e);
		}
		catch (IllegalTimeSeriesException e) {
		  System.err.println(e);
		}
		catch (Exception e)
		{
			System.err.println(e);
		}
	}
}
