package saaps.dbt;

import java.util.*;
import java.text.*;
import java.io.IOException;

import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.NotBoundException;
import java.net.MalformedURLException;

import saaps.util.*;


/**
 * Displays data from the selected remote Database.
 */
public class RemoteInspect {
  private String dbName;
  private DatabaseRMI dbrmi = null;
  private Vector dbList = null;
  private Calendar sd = Calendar.getInstance(TimeZone.getTimeZone("UTC")
					     ,new Locale("sv","SE"));
  private Calendar ed = Calendar.getInstance(TimeZone.getTimeZone("UTC"),
				       new Locale("sv","SE"));

  /**
   * Gives us an object with all input parameters initialized.
   * @param dbn Database name
   * @param RMIServer name of the remote server
   * @param dateTime start date/time
   * @param timeSpan data interval in minutes
   */
  public RemoteInspect(String dbn,String RMIServer,String dateTime,
			String timeSpan) {    
    try {
      dbrmi = (DatabaseRMI) Naming.lookup("//" + RMIServer +
					  ":2001" + "/DatabaseRMIServer");
      dbList = dbrmi.getDatabaseList();
    }
    catch(RemoteException re) {
      System.err.println(re);
    }
    catch(NotBoundException nbe) {
      System.err.println(nbe);
    }
    catch(MalformedURLException murl) {
      System.err.println(murl);
    }

    dbName = dbn;
    SimpleDateFormat formatter = (SimpleDateFormat)
      SimpleDateFormat.getDateTimeInstance(0,0,new Locale("sv","SE"));
    
    formatter.setTimeZone(TimeZone.getTimeZone("UTC"));
    formatter.applyPattern("yyyy.MM.dd HH:mm:ss");
    
    /* Parse the string back into a Date. */
    ParsePosition pos = new ParsePosition(0);
    Date inputDateTime = formatter.parse(dateTime, pos);
    
    /* Our selected start time */
    sd.setTime(inputDateTime);
    sd.clear(sd.MILLISECOND);
    /* our selected end time */
    ed.setTime(new Date ((long) sd.getTime().getTime() +
			 (new Long(timeSpan)).longValue()*60*1000));
    ed.clear(ed.MILLISECOND);
  }

  /**
   * Gets an <code>ObjectTimeSeries</code> object.
   */
  public ObjectTimeSeries goGet() throws 
			  IOException, IllegalTimeSeriesException {
    return dbrmi.retrieveOTS(dbName,sd.getTime(),ed.getTime(),true);
  }

  /**
   * Prints out the contents of the input ObjectTimeSeries.
   */
  public void display(ObjectTimeSeries ots) {
    for(int i=0;i<ots.getLength();i++) {
      System.out.println(((DatabaseObject) ots.getDataObject(i)).toString());
    }
  }

  /**
   * Main of RemoteInspect. Extracts the command line arguments and
   * runs the metods to get and  display data.
   */
  public static void main(String[] argv) {

    if(argv.length<5) {
      System.out.println("Usage: java saaps.dbt.RemoteInspect <Db class name> <RMIServer> <yyyy.mm.dd> <hh:mm:ss> <duration> \n   duration in minutes");
      System.exit(0);
    }

    String dbn = argv[0];
    String RMIServer = argv[1];
    String dateTime = new String(argv[2] + " " + argv[3]);
    String timeSpan = argv[4];

    RemoteInspect di = new RemoteInspect(dbn,RMIServer,dateTime,timeSpan);
    try {
      di.display(di.goGet());
    }
    catch (IOException ioe) {
      System.err.println(ioe);
    }
    catch (IllegalTimeSeriesException itse) {
      System.err.println(itse);
    }
  }
}

