package saaps.dbt;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import saaps.dbt.*;

/**
* LatestPrelKp downloads the latest preliminary Kp from SEC
* and stores it in the SAAPS database.
* FTP address: ftp://ftp.sel.noaa.gov/pub/latest/DGD.txt
*
* The application uses the single parameter databas class
* DatabaseSP and its related classes.
*
*@author Peter Wintoft
*@version 2.0 1999-09-22
*@version 1.0 990901
*/
public class TestPrelKp
{
  private static double dataGapValue = -1.0;

	public static void main(String[] args)
	{
		String ftpSrcStr="ftp://ftp.sel.noaa.gov/pub/latest/DGD.txt";
		DateFormat df;
		Date d;
		System.out.println(ftpSrcStr);
		try
		{
			// Open connection
			URL url = new URL(ftpSrcStr);
			System.out.println(url);
			URLConnection urlConn = url.openConnection();
                        urlConn.connect();
			// Input stream.
			BufferedReader streamIn = new BufferedReader(
			                            new InputStreamReader(
			                              urlConn.getInputStream()));
			String textLine = "";
			textLine = streamIn.readLine();
			// Go through the file for all lines starting with
			// "#", ":", or " ".
			while (!textLine.startsWith("1") && !textLine.startsWith("2"))
			{
			System.out.println(textLine);
				textLine = streamIn.readLine();
			}
			// Create a Vector of Kp objects.
			Vector kpv = new Vector(8*30);
			while ((textLine = streamIn.readLine()) != null)
			{
			System.out.println(textLine);
				// Read the date using - as the field separator.
				String dateStr = textLine.substring(0,4) + "-" +
				               		textLine.substring(5,7) + "-" +
				               		textLine.substring(8,10);
				df = DateFormat.getDateInstance(DateFormat.SHORT,new Locale("sv","SE"));
				df.setTimeZone(TimeZone.getTimeZone("UTC"));
				d = df.parse(dateStr);
				// Assume that observations are at 00, 03, 06, ...
				int h;
				Calendar cal = Calendar.getInstance();
				cal.setTimeZone(df.getTimeZone());
				for (int i=0;i<8;i++)
				{
					h=3*i;
					cal.setTime(d);
					cal.add(Calendar.HOUR,h);
					double tmpKp = Double.valueOf(textLine.substring(63+2*i,65+2*i).trim()).doubleValue();
					if (tmpKp == dataGapValue ) {
					  tmpKp = Double.NaN;
					}
					kpv.addElement(new Kp(cal.getTime().getTime(),tmpKp));
					// Integer.valueOf(textLine.substring(63+2*i,65+2*i).trim()).intValue()));
				}
			}
			// Trim the vector.
			kpv.trimToSize();
			// Close input stream.
			streamIn.close();
		}
		catch (IOException e)
		{
			System.err.println("URL Connection error: " + e);
		}
		catch (NumberFormatException e)
		{
			System.err.println("Number format error: " + e);
		}
		catch (ParseException e)
		{
			System.err.println("Parse error: " + e);
		}
	}
}
