/*
 * ProbCond.java
 *
 * Created on June 20, 2001, 9:01 PM
 */

package saaps.math;

import java.io.Serializable;

/**
 *
 * @author  root
 * @version 
 */
public class ProbCond extends Object implements Serializable {
    
    /**
     *The probability that X=x given that Y=y, or
     *P(X=x|Y=y).
     */
    public double xy[][];

    /**
     *The probability that Y=y given that X=x, or
     *P(Y=y|X=x).
     */
    public double yx[][];
    
    //The joint probability.
    Prob prob;
    
    public ProbCond() {
        this.prob = new Prob();
    }
    
    public ProbCond(int m, int n) {
        this.prob = new Prob(m,n);
        this.calcProbCond();
    }
    
    public ProbCond(Prob prob) {
        this.prob = prob;
        this.calcProbCond();
    }
    
    /** Creates new ProbCond */
    public void calcProbCond(Prob prob) {
        this.prob = prob;
        this.calcProbCond();
    }
    
    /**
     *Calculate the conditional probability for the two arrays x and y.
     *The number of classes in x and y are m=max(x)+1 and n=max(y)+1, respectively.
     *The xy field will thus have m rows and n columns.
     */
    public void calcProbCond(int[] x, int[] y) {
        this.prob.calcProb(x,y);
        this.calcProbCond();
    }
    
    /**
     *Calculate the conditional probability for the two arrays x and y.
     *The number of classes in x and y are set to m and n, respectively.
     *The xy field will thus have m rows and n columns.
     */
    public void calcProbCond(int[] x, int[] y, int m, int n) {
        this.prob.calcProb(x,y,m,n);
        this.calcProbCond();
    }
    
    public void calcProbCond() {
        this.xy = new double[this.prob.x.length][this.prob.y.length];
        this.yx = new double[this.prob.x.length][this.prob.y.length];
        for (int i=0;i<this.prob.x.length;i++) {
            for (int j=0;j<this.prob.y.length;j++) {
                this.xy[i][j] = this.prob.xy[i][j] / this.prob.y[j];
                this.yx[i][j] = this.prob.xy[i][j] / this.prob.x[i];
            }
        }
    }
    
    public void set(ProbCond pc) {
        for (int i=0;i<this.xy.length;i++) {
            for (int j=0;j<this.xy[i].length;j++) {
                this.xy[i][j] = pc.xy[i][j];
                this.yx[i][j] = pc.yx[i][j];
            }
        }
    }
    
    /**
    * @param args the command line arguments
    */
    public static void main (String args[]) {
        int[] x = {0,1,2,3,0,1,2,3,0,1,2,3};
        int[] y = {0,1,2,3,4,0,1,2,3,4,0,1};
        Prob p = new Prob(x,y);
        ProbCond pc = new ProbCond(p);
        double s = 0;
        for (int i=0;i<p.x.length;i++) {
            for (int j=0;j<p.y.length;j++) {
                System.out.println("pxy(" + i + "," + j + ")=" + pc.xy[i][j]);
                s += pc.xy[i][j];
            }
        }
        System.out.println("Sum pxy = " + s);
        s = 0;
        for (int i=0;i<p.x.length;i++) {
            for (int j=0;j<p.y.length;j++) {
                System.out.println("pyx(" + i + "," + j + ")=" + pc.yx[i][j]);
                s += pc.yx[i][j];
            }
        }
        System.out.println("Sum pyx = " + s);
    }

}
