/*
 * ElFluenceLevel.java
 *
 * Created on June 27, 2001, 10:18 AM
 */

package saaps.saam;

import java.util.Date;
import java.io.Serializable;
import java.io.IOException;

import saaps.math.ProbCond;
import saaps.util.EventList;

/**
 *
 * @author  root
 * @version 
 */
public class ElFluenceLevel extends Object implements Serializable {

    //The event list.
    private EventList eventList;
    //The non-event list.
    private EventList nonEventList;
    
    //The time delays in days. The first element is the
    //time delay for the 0.6 MeV and the second for the 2.0 MeV.
    private int[] timeDelay = new int[2];
    
    //The fluence levels in cm^-2 sr^-1.
    private double[] level = new double[2];
    
    //The conditional probabilities.
    private ProbCond[] prob = new ProbCond[2];
    
    /** Creates new ElFluenceLevel */
    public ElFluenceLevel() {
        //Init prob.
        prob[0] = new ProbCond(2,2);
        prob[1] = new ProbCond(2,2);
    }
    
    //////////////////////////
    // PUBLIC METHODS
    //////////////////////////
    
    public void setEventList(EventList el) {
        this.eventList = el;
    }
        
    public void setNonEventList(EventList el) {
        this.nonEventList = el;
    }
    
    public void setTimeDelay(int[] td) {
        for (int i=0;i<this.timeDelay.length;i++) this.timeDelay[i] = td[i];
    }

    public void setLevel(double[] l) {
        for (int i=0;i<this.level.length;i++) this.level[i] = l[i];
    }
    
    public int[] getTimeDelay() {
        return this.timeDelay;
    }
    
    public double[] getLevel() {
        return this.level;
    }
    
    public ProbCond[] getProbCond() {
        return this.prob;
    }
    
    public String get06Stat() {
        double pxy = ( this.prob[0].xy[0][0] + this.prob[0].xy[1][1] ) / 2.0;
        double pyx = ( this.prob[0].yx[0][0] + this.prob[0].yx[1][1] ) / 2.0;
        return new String("P(X|Y)=" + pxy + "\nP(Y|X)=" + pyx);
    }
    
    public String get20Stat() {
        double pxy = ( this.prob[1].xy[0][0] + this.prob[1].xy[1][1] ) / 2.0;
        double pyx = ( this.prob[1].yx[0][0] + this.prob[1].yx[1][1] ) / 2.0;
        return new String("P(X|Y)=" + pxy + "\nP(Y|X)=" + pyx);
    }
    
    public String get06OptText() {
        StringBuffer sb = new StringBuffer();
        sb.append("The time delay was varied from -5 to 5 days.");
        sb.append("\nThe 0.6 MeV electron fluence level was varied from 10^7 to 10^11 cm^-2 sr^-1.");
        sb.append("\nThe optimal time delay and threshold level gives the highest probability that the prediction is correct.");
        sb.append("\n\nThe optimal time delay = " + this.timeDelay[0] + " days.");
        sb.append("\nThe optimal threshold level = " + this.level[0] + " cm^-2 sr^-1.");
        double p = ( this.prob[0].xy[0][0] + this.prob[0].xy[1][1] ) / 2.0;
        sb.append("\n\nThe probability that the prediction is observed = " + p);
        p = ( this.prob[0].yx[0][0] + this.prob[0].yx[1][1] ) / 2.0;
        sb.append("\nThe probability that the observation is predicted = " + p);
        return sb.toString();
    }
    
    public String get20OptText() {
        StringBuffer sb = new StringBuffer();
        sb.append("The time delay was varied from -5 to 5 days.");
        sb.append("\nThe 2 MeV electron fluence level was varied from 10^5 to 10^9 cm^-2 sr^-1.");
        sb.append("\nThe optimal time delay and threshold level gives the highest probability that the prediction is correct.");
        sb.append("\n\nThe optimal time delay = " + this.timeDelay[1] + " days.");
        sb.append("\nThe optimal threshold level = " + this.level[1] + " cm^-2 sr^-1.");
        double p = ( this.prob[1].xy[0][0] + this.prob[1].xy[1][1] ) / 2.0;
        sb.append("\n\nThe probability that the prediction is observed = " + p);
        p = ( this.prob[1].yx[0][0] + this.prob[1].yx[1][1] ) / 2.0;
        sb.append("\nThe probability that the observation is predicted = " + p);
        return sb.toString();
    }
    
    public String get06RunText() {
        StringBuffer sb = new StringBuffer();
        sb.append("Time delay = " + this.timeDelay[0] + " days.");
        sb.append("\nThreshold level = " + this.level[0] + " cm^-2 sr^-1.");
        double p = ( this.prob[0].xy[0][0] + this.prob[0].xy[1][1] ) / 2.0;
        sb.append("\n\nThe probability that the prediction is observed = " + p);
        p = ( this.prob[0].yx[0][0] + this.prob[0].yx[1][1] ) / 2.0;
        sb.append("\nThe probability that the observation is predicted = " + p);
        return sb.toString();
    }
    
    public String get20RunText() {
        StringBuffer sb = new StringBuffer();
        sb.append("Time delay = " + this.timeDelay[1] + " days.");
        sb.append("\nThreshold level = " + this.level[1] + " cm^-2 sr^-1.");
        double p = ( this.prob[1].xy[0][0] + this.prob[1].xy[1][1] ) / 2.0;
        sb.append("\n\nThe probability that the prediction is observed = " + p);
        p = ( this.prob[1].yx[0][0] + this.prob[1].yx[1][1] ) / 2.0;
        sb.append("\nThe probability that the observation is predicted = " + p);
        return sb.toString();
    }
    
    /**
     *Use the current settings of event and non-event lists, time delays,
     *and levels and calculate the statistics for both energy levels.
     */
    public void calcStat() throws IOException {
        //Create the ElFluence object to be used to read fluence data.
        ElFluence ef = new ElFluence();
        //Init the array that hold the fluence.
        double[][] fluence = this.initFluence();
        //Init the array that hold the events.
        int[] events = this.initEvents();
        //Init the array that hold the predicted events.
        int[][] eventsPred = this.initEventsPred();
        //Init the conditional probabilities.
        ProbCond[] probCurr = this.initProbCond();

        //i=0 => 0.6 MeV
        //i=1 => 2.0 MeV
        for (int i=0;i<this.prob.length;i++) {
            //Read the fluence data with this time delay.
            this.readFluence(ef,this.timeDelay[i],fluence);
            //Get predicted events.
            this.getEventsPred(fluence,this.level,eventsPred);
            //Calculate the conditional probability.
            this.calcProbCond(events,eventsPred,probCurr);
            this.prob[i] = probCurr[i];
        }
    }
    
    /**
     *Vary the time delays from -5 to +5 days, the 0.6 MeV fluence levels
     *from 10^7 to 10^11, and the 2.0 MeV fluence levels from 10^5 to 10^9.
     *The fluence levels are varied in steps of 10^0.5. Find the time delay
     *and level that maximizes the conditional probability that the
     *observed events are correctly predicted, i.e. optimize with
     *respect to P(prediced|observed).
     */
    public void optimize() throws IOException {
        //Create the ElFluence object to be used to read fluence data.
        ElFluence ef = new ElFluence();
        //Init the array that hold the events.
        int[] events = this.initEvents();
        //Init the array that hold the fluence.
        double[][] fluence = this.initFluence();
        //Init the array that hold the predicted events.
        int[][] eventsPred = this.initEventsPred();
        //Init the conditional probabilities.
        ProbCond[] probCurr = this.initProbCond();
        //The levels.
//        double[] lev = new double[2];
        double[][] levs = new double[10][2];
        levs[0][1]=1e5; levs[1][1]=3e5;
        levs[2][1]=1e6; levs[3][1]=3e6; 
        levs[4][1]=1e7; levs[5][1]=3e7;
        levs[6][1]=1e8; levs[7][1]=3e8; 
        levs[8][1]=1e9; levs[9][1]=3e9;
        for (int i=0;i<levs.length;i++) levs[i][0] = levs[i][1]*100.0;
        //The time delays.
        int[] tds = new int[7];
        tds[0]=0; tds[1]=-1; tds[2]=+1; tds[3]=-2;
        tds[4]=+2; tds[5]=-3; tds[6]=+3;
        //The levels.
        for (int tdi=0;tdi<tds.length;tdi++) {
            int td = tds[tdi];
System.out.println("=============================================");
System.out.println("td=" + td);
            //Read the fluence data with this time delay.
            this.readFluence(ef,td,fluence);
            //The levels
//            for (double levPot=5;levPot<10;levPot+=0.5) {
            for (int levsi=0;levsi<levs.length;levsi++) {
                //The 0.6 MeV level.
//                lev[0] = java.lang.Math.rint(java.lang.Math.pow(10.0,levPot+2.0)*10)/10;
                //The 2.0 MeV level.
//                lev[1] = java.lang.Math.rint(java.lang.Math.pow(10.0,levPot)*10)/10;
                System.out.println("..............................................");
System.out.println("0.6 level = " + levs[levsi]);
System.out.println("2.0 level = " + levs[levsi]);
                //Get the predicted events using this threshold level.
                this.getEventsPred(fluence,levs[levsi],eventsPred);
                //Calculate the conditional probability.
                this.calcProbCond(events,eventsPred,probCurr);
                //Use current time delay and level if optimal.
                this.optimal(probCurr,td,levs[levsi]);
System.out.println("Optimal 0.6 td = " + this.timeDelay[0]);
System.out.println("Optimal 2.0 td = " + this.timeDelay[1]);
System.out.println("Optimal 0.6 level = " + this.level[0]);
System.out.println("Optimal 2.0 level = " + this.level[1]);
            }
        }
    }
    
    //////////////////////////////
    // PRIVATE METHODS
    //////////////////////////////
    
    private int[] initEvents() {
        //The total number of events equals the sum of the number
        //of events and non-events.
        int[] e = new int[this.eventList.getNumOfEvents() +
                                this.nonEventList.getNumOfEvents()];
        //Event index.
        int i = 0;
        //Loop through the events.
        for (int j=0;j<this.eventList.getNumOfEvents();j++) {
            //Let events equal 1.
            e[i++] = 1;
        }
        for (int j=0;j<this.nonEventList.getNumOfEvents();j++) {
            e[i++] = 0;
        }
        return e;
    }
    
    private double[][] initFluence() {
        double[][] f = new double[2][this.eventList.getNumOfEvents() +
                                        this.nonEventList.getNumOfEvents()];
        return f;
    }
    
    private int[][] initEventsPred() {
        int[][] e = new int[2][this.eventList.getNumOfEvents() +
                                this.nonEventList.getNumOfEvents()];
        return e;
    }
    
    private ProbCond[] initProbCond() {
        ProbCond[] p = new ProbCond[2];
        p[0] = new ProbCond();
        p[1] = new ProbCond();
        return p;
    }
    
    private void readFluence(ElFluence ef, int td, double[][] f) throws IOException {
        //Index to f.
        int i = 0;
        //The date for reading the fluence.
        Date d = new Date();
        //The time delay in ms.
        long tdms = ((long) td) * 86400000L;
        //Go through the events.
        for (int j=0;j<this.eventList.getNumOfEvents();j++) {
            //Get the date and add time delay.
            d.setTime(this.eventList.getEvent(j).getTime() + tdms);
            ef.setDate(d);
            //Get the fluences.
            f[0][i] = ef.get06Fluence();
            f[1][i++] = ef.get20Fluence();
        }
System.out.println("One observed 0.6 event fluence=" + f[0][i-1]);
System.out.println("One observed 2.0 event fluence=" + f[1][i-1]);
        //Go through the non-events.
        for (int j=0;j<this.nonEventList.getNumOfEvents();j++) {
            //Get the date and add time delay.
            d.setTime(this.nonEventList.getEvent(j).getTime() + tdms);
            ef.setDate(d);
            //Get the fluences.
            f[0][i] = ef.get06Fluence();
            f[1][i++] = ef.get20Fluence();
        }
System.out.println("One observed 0.6 non-event fluence=" + f[0][i-1]);
System.out.println("One observed 2.0 non-event fluence=" + f[1][i-1]);
    }
    
    private void getEventsPred(double[][] fluence, double[] lev, int[][] eventsPred) {
        for (int i=0;i<fluence.length;i++) {
            for (int j=0;j<fluence[i].length;j++) {
                if (fluence[i][j] > lev[i])
                    eventsPred[i][j] = 1;
                else
                    eventsPred[i][j] = 0;
            }
        }
    }
    
    private void calcProbCond(int[] events, int[][] eventsPred, ProbCond[] probCurr) {
        for (int i=0;i<probCurr.length;i++) {
            probCurr[i].calcProbCond(events,eventsPred[i],2,2);
        }
    }
    
    private void optimal(ProbCond[] probCurr,int td, double[] lev) {
        for (int i=0;i<probCurr.length;i++) {
            double oldProb = ( this.prob[i].yx[0][0] + this.prob[i].yx[1][1] ) / 2.0;
            double newProb = ( probCurr[i].yx[0][0] + probCurr[i].yx[1][1] ) / 2.0;
System.out.println("oldProb=" + oldProb);
System.out.println("newProb=" + newProb);
            //Is the new probability higher than the old?
            if (newProb > oldProb) {
                this.timeDelay[i] = td;
                this.level[i] = lev[i];
                this.prob[i].set(probCurr[i]);
            }
        }
    }
}
