/*
 * ElFluence.java
 *
 * Created on June 27, 2001, 8:39 AM
 */

package saaps.saam;

import java.io.IOException;
import java.util.Date;

//import saaps.dbt.Database;
//import saaps.dbt.KpDB;
import saaps.dbt.ODIDatabase;
import saaps.util.TimeSeries;

/**
 *
 * @author  root
 * @version 
 */
public class SumKp extends Object {

    //The database.
//    private KpDB db = (KpDB) Database.getInstance("KpDB");
    private ODIDatabase odiDatabase;
    private String sedatdbname = "KPAP_3h";
    
    //The sum Kp.
    private double sumKp;
    
    //The Kp index. There are 8 three-hour values per day.
    private double[] kp = new double[8];
    
    //The number of Kp values might be less than 8 if
    //there are NaN's in the data. This integer contain the
    //number of Kp values that is actually present.
    private int numOfKp;
    
    /** Creates new ElFluence */
    public SumKp() {
	this.odiDatabase = new ODIDatabase();
    }
    
    ////////////////////////////
    // PUBLIC METHODS
    ////////////////////////////
    
    /**
     *Set the date, load the GOES-08 0.6 MeV and 2.0 MeV
     *electron fluxes, and calculate the one day fluence.
     *
     *@param d The date for which the fluence should be calculated.
     */
    public void setDate(Date d) {
        //Read the flux.
        this.readKp(d);
        //Calculate the fluence.
        this.calcSumKp();
    }
    
    /**
     *Get the 0.6 MeV electron fluence. The unit is cm^-2 sr^-1.
     */
    public double getSumKp() {
        return this.sumKp;
    }
    

    ///////////////////////////////
    // PRIVATE METHODS
    ///////////////////////////////
    
    /**
     *Read Kp.
     */
    private void readKp(Date d1) {
        //The flux.
        TimeSeries ts = null;
        //The end date, 24 hours after the start date.
        //24 hours = 24*60*60*1000 ms = 86400000 ms.
        Date d2 = new Date(d1.getTime() + 86400000L);
        //System.out.println("d1:"+d1+" d2:"+d2);
        try {
//            ts = db.retrieveTS(d1,d2,true);
            ts = odiDatabase.retrieveTS(sedatdbname,d1,d2);
        }
        catch (Exception e) {
            System.err.println(e);
        }
        this.numOfKp = 0;
        //Get the smallest of the ts length and the flux length.
        int imax = this.kp.length;
        if (imax > ts.getLength()) imax = ts.getLength();
        //Extract the data.
        for (int i=0;i<imax;i++) {
            //Kp
            if (!Double.isNaN(ts.getData(i,0))) {
                this.numOfKp++;
                this.kp[i] = ts.getData(i,0);
            }
        }
    }
    
    private void calcSumKp() {
        this.sumKp = 0;
        //Sum the Kp data.
        for (int j=0;j<this.numOfKp;j++) {
            this.sumKp += this.kp[j];
        }
        //Normalize the sum if there are less than eight values.
        if (this.numOfKp>0) {
            this.sumKp *= ( 8.0 / this.numOfKp );
        }
        else {
            this.sumKp = Double.NaN;
        }
    }
    
    /**
    * @param args the command line arguments
    */
    public static void main (String args[]) {
        if (args.length<6) {
            System.out.println("Usage: SumKp <year> <month> <date> <hour> <minute> <second>");
        }
        int[] date = new int[6];
        for (int i=0;i<date.length;i++) {
            date[i] = Integer.parseInt(args[i]);
        }
        date[1]--;
        java.util.TimeZone tz = java.util.TimeZone.getTimeZone("GMT");
        java.util.Calendar cal = java.util.Calendar.getInstance(tz);
        cal.set(java.util.Calendar.YEAR,date[0]);
        cal.set(java.util.Calendar.MONTH,date[1]);
        cal.set(java.util.Calendar.DATE,date[2]);
        cal.set(java.util.Calendar.HOUR_OF_DAY,date[3]);
        cal.set(java.util.Calendar.MINUTE,date[4]);
        cal.set(java.util.Calendar.SECOND,date[5]);
        cal.set(java.util.Calendar.MILLISECOND,0);
        SumKp sk = new SumKp();
        sk.setDate(cal.getTime());
        System.out.println("Number of Kp values = " + sk.numOfKp);
        System.out.println("Sum Kp = " + sk.getSumKp());
    }

}
