/*
 * SumKpLevel.java
 *
 * Created on June 27, 2001, 10:18 AM
 */

package saaps.saam;

import java.util.Date;
import java.io.Serializable;

import saaps.math.ProbCond;
import saaps.util.EventList;

/**
 *
 * @author  root
 * @version 
 */
public class SumKpLevel extends Object implements Serializable {

    //The event list.
    private EventList eventList;
    //The non-event list.
    private EventList nonEventList;
    
    //The time delays in days. The first element is the
    //time delay for the 0.6 MeV and the second for the 2.0 MeV.
    private int timeDelay;
    
    //The fluence levels in cm^-2 sr^-1.
    private double level;
    
    //The conditional probabilities.
    private ProbCond prob = new ProbCond(2,2);
    
    /** Creates new ElFluenceLevel */
    public SumKpLevel() {
    }
    
    //////////////////////////
    // PUBLIC METHODS
    //////////////////////////
    
    public void setEventList(EventList el) {
        this.eventList = el;
    }
        
    public void setNonEventList(EventList el) {
        this.nonEventList = el;
    }
    
    public void setTimeDelay(int td) {
        this.timeDelay = td;
    }

    public void setLevel(double l) {
        this.level = l;
    }
    
    public int getTimeDelay() {
        return this.timeDelay;
    }
    
    public double getLevel() {
        return this.level;
    }
    
    public ProbCond getProbCond() {
        return this.prob;
    }
    
/*    public String get06Stat() {
        double pxy = ( this.prob[0].xy[0][0] + this.prob[0].xy[1][1] ) / 2.0;
        double pyx = ( this.prob[0].yx[0][0] + this.prob[0].yx[1][1] ) / 2.0;
        return new String("P(X|Y)=" + pxy + "\nP(Y|X)=" + pyx);
    }
    
    public String get20Stat() {
        double pxy = ( this.prob[1].xy[0][0] + this.prob[1].xy[1][1] ) / 2.0;
        double pyx = ( this.prob[1].yx[0][0] + this.prob[1].yx[1][1] ) / 2.0;
        return new String("P(X|Y)=" + pxy + "\nP(Y|X)=" + pyx);
    }*/
    
    public String getOptText() {
        StringBuffer sb = new StringBuffer();
        sb.append("The time delay was varied from -5 to 5 days.");
        sb.append("\nThe sum Kp level was varied from 0 to 72.");
        sb.append("\nThe optimal time delay and threshold level gives the highest probability that the prediction is correct.");
        sb.append("\n\nThe optimal time delay = " + this.timeDelay + " days.");
        sb.append("\nThe optimal threshold level = " + this.level );
        double p = ( this.prob.xy[0][0] + this.prob.xy[1][1] ) / 2.0;
        sb.append("\n\nThe probability that the prediction is observed = " + p);
        p = ( this.prob.yx[0][0] + this.prob.yx[1][1] ) / 2.0;
        sb.append("\nThe probability that the observation is predicted = " + p);
        return sb.toString();
    }
    

    public String getRunText() {
        StringBuffer sb = new StringBuffer();
        sb.append("Time delay = " + this.timeDelay + " days.");
        sb.append("\nThreshold level = " + this.level );
        double p = ( this.prob.xy[0][0] + this.prob.xy[1][1] ) / 2.0;
        sb.append("\n\nThe probability that the prediction is observed = " + p);
        p = ( this.prob.yx[0][0] + this.prob.yx[1][1] ) / 2.0;
        sb.append("\nThe probability that the observation is predicted = " + p);
        return sb.toString();
    }
        
    /**
     *Use the current settings of event and non-event lists, time delays,
     *and levels and calculate the statistics for both energy levels.
     */
    public void calcStat() {
        //Create the SumKp object to be used to read Kp data.
        SumKp sk = new SumKp();
        //Init the array that hold the sum Kp.
        double[] sumKp = this.initSumKp();
        //Init the array that hold the events.
        int[] events = this.initEvents();
        //Init the array that hold the predicted events.
        int[] eventsPred = this.initEventsPred();
        //Init the conditional probabilities.
        ProbCond probCurr = this.initProbCond();

        //Read the sum Kp data with this time delay.
        this.readSumKp(sk,this.timeDelay,sumKp);
        //Get predicted events.
        this.getEventsPred(sumKp,this.level,eventsPred);
        //Calculate the conditional probability.
        this.calcProbCond(events,eventsPred,probCurr);
        this.prob = probCurr;
    }
    
    /**
     *Vary the time delays from -5 to +5 days, the 0.6 MeV fluence levels
     *from 10^7 to 10^11, and the 2.0 MeV fluence levels from 10^5 to 10^9.
     *The fluence levels are varied in steps of 10^0.5. Find the time delay
     *and level that maximizes the conditional probability that the
     *observed events are correctly predicted, i.e. optimize with
     *respect to P(prediced|observed).
     */
    public void optimize() {
        //Create the SumKp object to be used to read Kp data.
        SumKp sk = new SumKp();
        //Init the array that hold the events.
        int[] events = this.initEvents();
        //Init the array that hold the sum Kp.
        double[] sumKp = this.initSumKp();
        //Init the array that hold the predicted events.
        int[] eventsPred = this.initEventsPred();
        //Init the conditional probabilities.
        ProbCond probCurr = this.initProbCond();
        //The levels.
//        double[] lev = new double[2];
        double[] levs = new double[8];
        levs[0] = 0.0;
        levs[1] = 10.0;
        levs[2] = 20.0;
        levs[3] = 30.0;
        levs[4] = 40.0;
        levs[5] = 50.0;
        levs[6] = 60.0;
        levs[7] = 70.0;
        //The time delays.
        int[] tds = new int[7];
        tds[0]=0; tds[1]=-1; tds[2]=+1; tds[3]=-2;
        tds[4]=+2; tds[5]=-3; tds[6]=+3;
        //The levels.
        for (int tdi=0;tdi<tds.length;tdi++) {
            int td = tds[tdi];
System.out.println("=============================================");
System.out.println("td=" + td);
            //Read the sum Kp data with this time delay.
            this.readSumKp(sk,td,sumKp);
            //The levels
            for (int levsi=0;levsi<levs.length;levsi++) {
                //The 0.6 MeV level.
//                lev[0] = java.lang.Math.rint(java.lang.Math.pow(10.0,levPot+2.0)*10)/10;
                //The 2.0 MeV level.
//                lev[1] = java.lang.Math.rint(java.lang.Math.pow(10.0,levPot)*10)/10;
                System.out.println("..............................................");
System.out.println("Level = " + levs[levsi]);
                //Get the predicted events using this threshold level.
                this.getEventsPred(sumKp,levs[levsi],eventsPred);
                //Calculate the conditional probability.
                this.calcProbCond(events,eventsPred,probCurr);
                //Use current time delay and level if optimal.
                this.optimal(probCurr,td,levs[levsi]);
System.out.println("Optimal td = " + this.timeDelay);
System.out.println("Optimal level = " + this.level);
            }
        }
    }
    
    //////////////////////////////
    // PRIVATE METHODS
    //////////////////////////////
    
    private int[] initEvents() {
        //The total number of events equals the sum of the number
        //of events and non-events.
        int[] e = new int[this.eventList.getNumOfEvents() +
                                this.nonEventList.getNumOfEvents()];
        //Event index.
        int i = 0;
        //Loop through the events.
        for (int j=0;j<this.eventList.getNumOfEvents();j++) {
            //Let events equal 1.
            e[i++] = 1;
        }
        for (int j=0;j<this.nonEventList.getNumOfEvents();j++) {
            e[i++] = 0;
        }
        return e;
    }
    
    private double[] initSumKp() {
        double[] f = new double[this.eventList.getNumOfEvents() +
                                        this.nonEventList.getNumOfEvents()];
        return f;
    }
    
    private int[] initEventsPred() {
        int[] e = new int[this.eventList.getNumOfEvents() +
                                this.nonEventList.getNumOfEvents()];
        return e;
    }
    
    private ProbCond initProbCond() {
        ProbCond p = new ProbCond();
        return p;
    }
    
    private void readSumKp(SumKp sk, int td, double[] f) {
        //Index to f.
        int i = 0;
        //The date for reading the fluence.
        Date d = new Date();
        //The time delay in ms.
        long tdms = ((long) td) * 86400000L;
        //Go through the events.
        for (int j=0;j<this.eventList.getNumOfEvents();j++) {
            //Get the date and add time delay.
            d.setTime(this.eventList.getEvent(j).getTime() + tdms);
            sk.setDate(d);
            //Get the fluences.
            f[i++] = sk.getSumKp();
        }
        //Go through the non-events.
        for (int j=0;j<this.nonEventList.getNumOfEvents();j++) {
            //Get the date and add time delay.
            d.setTime(this.nonEventList.getEvent(j).getTime() + tdms);
            sk.setDate(d);
            //Get the fluences.
            f[i++] = sk.getSumKp();
        }
    }
    
    private void getEventsPred(double[] sumKp, double lev, int[] eventsPred) {
            for (int j=0;j<sumKp.length;j++) {
                if (sumKp[j] > lev)
                    eventsPred[j] = 1;
                else
                    eventsPred[j] = 0;
            }
    }
    
    private void calcProbCond(int[] events, int[] eventsPred, ProbCond probCurr) {
            probCurr.calcProbCond(events,eventsPred,2,2);
    }
    
    private void optimal(ProbCond probCurr,int td, double lev) {
            double oldProb = ( this.prob.yx[0][0] + this.prob.yx[1][1] ) / 2.0;
            double newProb = ( probCurr.yx[0][0] + probCurr.yx[1][1] ) / 2.0;
System.out.println("oldProb=" + oldProb);
System.out.println("newProb=" + newProb);
            //Is the new probability higher than the old?
            if (newProb > oldProb) {
                this.timeDelay = td;
                this.level = lev;
                this.prob.set(probCurr);
            }
    }
}
