/*
 * LocalGOES08.java
 *
 * Created on October 20, 2000, 11:10 AM
 */
 
package saaps.util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/** 
 *  Create an ASCII file from the local GOES-08 data that is formatted
 *  so that it can be imported to the SAAPS database.
 *
 *  @author   Peter Wintoft 2000-10-20
 *  @version  1.0 
 */
public class LocalGOES08 extends Object {

  //The path of the local GOES-08 data relative to the SAAPS home directory.
  private String path = "/space/saaps/DatabaseLocal/";
  
  //The date formatter for the input.
  private SimpleDateFormat formatIn = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss:SSS");
  
  //The date formatter for the output.
  private SimpleDateFormat formatOut = new SimpleDateFormat("yyyy MM dd HH mm ss ");
  
  /** Creates new LocalGOES08 */
  public LocalGOES08() {
  }
  
  private BufferedReader openReader(int year)
  {
    String fileName = new String(path + "g8_" + year + "_ele.txt");
    BufferedReader buf = null;
    try
    {
      buf = new BufferedReader(new FileReader(fileName));
    }
    catch(FileNotFoundException fnfe)
    {
      System.err.println(fnfe);
    }
    return buf;
  }
    
  private String parseLine(String textIn) throws ParseException
  {
    //Get the date string.
    String dateStr = textIn.substring(0,24);
    //Get the e-flux string.
    String fluxStr = textIn.substring(24);
    //Create a string buffer.
    StringBuffer strBuf = new StringBuffer();
    //Create the formatted date string.
    Date d = formatIn.parse(dateStr);
    strBuf.append(formatOut.format(d));
    //Add NaNs for the missing proton data.
    strBuf.append("NaN NaN NaN NaN NaN NaN ");
    //Add the >0.6 and >2 MeV e-flux data.
    strBuf.append(fluxStr);
    //Add the missing >4 MeV e-flux data and the longitude.
    strBuf.append("NaN 75.0");
    return new String(strBuf);
  }
  
  private void addNaNs() throws ParseException
  {
    //Five minutes in milliseconds.
    long fiveMin = (long) 5*60*1000;
    //Start date.
    long t = formatIn.parse("2000/01/01 00:00:00:000").getTime();
    //End date.
    long t2 = formatIn.parse("2000/03/31 23:55:00:000").getTime();
    //Go through the dates.
    while (t<=t2)
    {
      System.out.println(formatOut.format(new Date(t)) + "NaN NaN NaN NaN NaN NaN NaN NaN NaN NaN");
      //Add 5 minutes.
      t += fiveMin;
    }
  }
  
  public void run()
  {
    String textLineIn;
    String textLineOut;
    try
    {
      for (int year=95;year<100;year++)
      {
        BufferedReader bufIn = openReader(year);
        //Skip comment lines.
        while(!(textLineIn=bufIn.readLine()).startsWith("199"));
        while((textLineIn=bufIn.readLine()) != null)
        {
          textLineOut = parseLine(textLineIn);
          System.out.println(textLineOut);
        }
        bufIn.close();
      }
      //Add NaNs for Jan-March 2000.
      addNaNs();
    }
    catch(ParseException pe)
    {
      System.err.println(pe);
    }
    catch(IOException ioe)
    {
      System.err.println(ioe);
    }
  }
  
  /**
  * Read the data from local file.
  /**
  * @param args the command line arguments
  */
  public static void main (String args[]) {
    //Create the LocalGOES08 object.
    LocalGOES08 lg08 = new LocalGOES08();
    //Read and write the data one year at a time.
    lg08.run();
  }

}