/*
 * Model.java
 *
 * Created on June 18, 2001, 8:09 AM
 */

package saaps.util;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Serializable;
import java.util.Date;

/**
 * This is an abtstract class for models. All models should extend this
 * class to ensure that certain methods are implemented.
 *
 * @author  Peter Wintoft
 * @version 1.0
 */
public abstract class Model extends java.lang.Object implements Serializable {

    //The model base directory variable.
    protected static String baseDirVar = "SAAPS_HOME";
    
    //The model base directory.
    protected static String baseDir = null;
    
    //The model name.
    protected String modelName;
    
    //The model info.
    protected String modelInfo;
    
    /**
     Set up the models directory from the system property
     variable <code>saaps.models</code>.
     */
    public Model() {
        this(true);
        //Set up the models base directory.
//        this.baseDir = System.getProperty(this.baseDirVar);
//        System.out.println(this.baseDirVar);
//        System.out.println(this.baseDir);
        //Ensure that the base directory ends with an "/".
//        if(!this.baseDir.endsWith("/")) {
//            this.baseDir += "/";
//        }
    }

    public Model(boolean readFromFile) {
        if (readFromFile) {
            this.baseDir = System.getenv(this.baseDirVar);
            //Ensure that the base directory ends with an "/".
            if(!this.baseDir.endsWith("/")) {
                this.baseDir += "/";
            }
            this.baseDir += "Models/";
//System.out.println(this.baseDir);
        }
    }       
    
    public abstract void setPredictionTime(Date predDate);

    public abstract void readTimeSeries(Date d1, Date d2);
    
    public abstract void run();
    
    public abstract TimeSeries getOutput();
    
    public String getName() {
        return this.modelName;
    }

    public void readInfo(String model) {
        try {
            FileReader file = new FileReader(this.baseDir + model);
            BufferedReader buf = new BufferedReader(file);
            this.modelInfo = "";
            String textLine = null;
            while ((textLine = buf.readLine()) != null)
                this.modelInfo += textLine + "\n";
        }
        catch (FileNotFoundException e) {
            System.err.println(e);
        }
        catch (IOException e) {
            System.err.println(e);
        }
    }
    
    public String getInfo() {
        return this.modelInfo;
    }

}
