/*
 * DataPlotterFrame.java
 *
 * Created on August 4, 2000, 3:02 PM
 */
 
package saaps.util;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;
import java.awt.PrintJob;
import java.awt.Rectangle;
import java.util.Date;

/** 
* This windows shows the plot requested from the Data Plotter Applet. The window
* contains a menu from which the plot axes and lines can be modified.
*
* @author  Peter Wintoft
* @version 1.0 2000-08-08
* @see      DataPlotterAppletAWT
* @see      DataPlotterAxisDialog
* @see      TimeSeries
* @see      TimeSeriesPlot.
*/
public class PlotFrame extends java.awt.Frame {

  //The plot surface. TimeSeriesPlot extends ptolemy.plot.Plot which
  //in turn extends java.awt.Panel.
  protected TimeSeriesPlot timeSeriesPlot;
  
  /** Creates new form DataPlotterFrame */
  public PlotFrame() {
    initComponents ();
    pack ();
  }

  /**
  * Plot all data fields in the <code>TimeSeries</code> object
  * as a function of calendar time.
  *
  * @param  ts The time series to be plotted.
  */
  public PlotFrame(TimeSeries ts) {
    this(new TimeSeriesPlot(ts));
  }

  /**
  * Plot a time series plot.
  *
  *@param tsp The time series plot
  */
  public PlotFrame(TimeSeriesPlot tsp)
  {
    //Plot the data.
    timeSeriesPlot = tsp;
    timeSeriesPlot.plotData();
    //Initialize the frame components such as
    //menus and menu items.
    initComponents();
    //Add the time series plot to the frame.
    add(timeSeriesPlot);
    //Set the menu states from the time series plot.
    checkboxMenuItemXAxis.setState(timeSeriesPlot.getXLog());
    checkboxMenuItemYAxis.setState(timeSeriesPlot.getYLog());
    pack();
  }
  
  public PlotFrame(TimeSeries ts, int xf, int[] yfs, boolean li, boolean po, boolean xLog, boolean yLog)
  {
    pack();
    //Plot the data.
    timeSeriesPlot = new TimeSeriesPlot(ts,xf,yfs,li,po,xLog,yLog);
    //Initialize the frame components such as
    //menus and menu items.
    initComponents();
    //Add the time series plot to the frame.
    add(timeSeriesPlot);
    //Set the menu states from the time series plot.
    checkboxMenuItemXAxis.setState(timeSeriesPlot.getXLog());
    checkboxMenuItemYAxis.setState(timeSeriesPlot.getYLog());
    pack();
  }
  
  /**
  * Plot selected data fields from a <code>TimeSeries</code> object as a function of
  * another field, which can be the time field or another data field.
  *
  * @param  ts  The time series object.
  * @param  xf  The field for the x-axis. This can be a data field (xf>=0) or
  *             <code>xf=TimeSeriesPlot.REL_TIME</code> or <code>xf=TimeSeriesPlot.CAL_TIME</code>.
  * @param  yfs The fields for the y-axis.
  * @param  li  True if lines should be used.
  * @param  po  True if points should be used.
  * @param  xLabel  The x-label.
  * @param  yLabels The y-labels.
  * @param  xLog  True if the x-axis should be logarithmic.
  * @param  yLog  True if the y-axis should be logarithmic.
  */
  public PlotFrame(TimeSeries ts,int xf,int[] yfs,boolean li,boolean po,String xLabel,String[] yLabels,boolean xLog,boolean yLog) {
    this(new TimeSeriesPlot(ts,xf,yfs,li,po,xLabel,yLabels,xLog,yLog));
//    initComponents ();
//    pack ();
    //Plot the data.
//    TimeSeriesPlot tsp = new TimeSeriesPlot(ts,xf,yfs,li,po,xLabel,yLabels,xLog,yLog);
//    tsp.plotData();
//    showPlot(tsp);
//    checkboxMenuItemXAxis.setState(tsp.getXLog());
//    checkboxMenuItemYAxis.setState(tsp.getYLog());
//    show();
//    checkboxMenuItemLines.setState(tsp.lines);
//    checkboxMenuItemPoints.setState(tsp.points);
  }

//  protected void showPlot(TimeSeriesPlot plot)
//  {
//    if (panel.getComponentCount() > 0)
//      panel.remove(0);
//    panel.add(plot,0);
//    panel.setSize(plot.getSize());
//    panel.validate();
//    plot.repaint();
//    pack();
//  }

  public void useCalTime()
  {
//    TimeSeriesPlot tsp = (TimeSeriesPlot) panel.getComponent(0);
    timeSeriesPlot.useCalTime();
//    showPlot(tsp);
  }
  
  public void useRelTime()
  {
//    TimeSeriesPlot tsp = (TimeSeriesPlot) panel.getComponent(0);
    timeSeriesPlot.useRelTime();
//    showPlot(tsp);
  }
  
  protected void addToMenuBar(java.awt.Menu m)
  {
    menuBar.add(m);
  }
  
  public void plotData()
  {
    timeSeriesPlot.plotData();
  }

  /**
   * This method will create a print job and print the plot.
   */
  protected void _print() {
      // The awt uses properties to set the defaults:
      // awt.print.destination   - can be "printer" or "file"
      // awt.print.printer       - print command
      // awt.print.fileName      - name of the file to print
      // awt.print.numCopies     - obvious
      // awt.print.options       - options to pass to the print command
      // awt.print.orientation   - can be "portrait" or "landscape"
      // awt.print.paperSize     - can be "letter", "legal", "executive"
      //                           or "a4"
      
      // Accept the defaults... But if you want to change them,
      // do something like this...
      java.util.Properties newprops = new java.util.Properties();
      newprops.put("awt.print.destination", "file");
      newprops.put("awt.print.fileName", "saapsplot.ps");
      newprops.put("awt.print.paperSize", "a4");
      newprops.put("awt.print.orientation", "landscape");

      PrintJob printjob = getToolkit().getPrintJob(this,
      getTitle(), newprops);
      //PrintJob printjob = getToolkit().getPrintJob(this,
      //getTitle(), null);
      if (printjob != null) {
          try {
              Graphics printgraphics = printjob.getGraphics();
              if (printgraphics != null) {
                  // Print only the plot frame.
                  try {
                      timeSeriesPlot.printAll(printgraphics);
                  } finally {
                      printgraphics.dispose();
                  }
              }
          } finally {
              printjob.end();
          }
      }
  }
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the FormEditor.
   */
  private void initComponents() {//GEN-BEGIN:initComponents
      menuBar = new java.awt.MenuBar();
      menuFile = new java.awt.Menu();
      menuItemPrint = new java.awt.MenuItem();
      menuItemQuit = new java.awt.MenuItem();
      menuAxis = new java.awt.Menu();
      checkboxMenuItemXAxis = new java.awt.CheckboxMenuItem();
      checkboxMenuItemYAxis = new java.awt.CheckboxMenuItem();
      menuItemAxisFill = new java.awt.MenuItem();
      menuItemSetAxesLimits = new java.awt.MenuItem();
      menuItemAxisCopy = new java.awt.MenuItem();
      menuItemAxisPaste = new java.awt.MenuItem();
      menuLines = new java.awt.Menu();
      checkboxMenuItemLines = new java.awt.CheckboxMenuItem();
      checkboxMenuItemPoints = new java.awt.CheckboxMenuItem();
      
      menuFile.setName("menuFile");
        menuFile.setLabel("File");
        
        menuItemPrint.setName("menuitemPrint");
          menuItemPrint.setLabel("Print ...");
          menuItemPrint.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(java.awt.event.ActionEvent evt) {
                  menuItemPrintActionPerformed(evt);
              }
          }
          );
          menuFile.add(menuItemPrint);
          
        menuFile.addSeparator();
          
        menuItemQuit.setName("menuitemQuit");
          menuItemQuit.setLabel("Quit");
          menuItemQuit.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(java.awt.event.ActionEvent evt) {
                  menuItemQuitActionPerformed(evt);
              }
          }
          );
          menuFile.add(menuItemQuit);
          menuBar.add(menuFile);
        
      menuAxis.setName("menuAxis");
        menuAxis.setLabel("Axis");
        
        checkboxMenuItemXAxis.setName("chkmenuitemXAxis");
          checkboxMenuItemXAxis.setLabel("X Log");
          checkboxMenuItemXAxis.addItemListener(new java.awt.event.ItemListener() {
              public void itemStateChanged(java.awt.event.ItemEvent evt) {
                  checkboxMenuItemXAxisItemStateChanged(evt);
              }
          }
          );
          menuAxis.add(checkboxMenuItemXAxis);
          
        checkboxMenuItemYAxis.setName("chkmenuitemYAxis");
          checkboxMenuItemYAxis.setLabel("Y Log");
          checkboxMenuItemYAxis.addItemListener(new java.awt.event.ItemListener() {
              public void itemStateChanged(java.awt.event.ItemEvent evt) {
                  checkboxMenuItemYAxisItemStateChanged(evt);
              }
          }
          );
          menuAxis.add(checkboxMenuItemYAxis);
          
        menuAxis.addSeparator();
          
        menuItemAxisFill.setLabel("Fill");
          menuItemAxisFill.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(java.awt.event.ActionEvent evt) {
                  menuItemAxisFillActionPerformed(evt);
              }
          }
          );
          menuAxis.add(menuItemAxisFill);
          
        menuItemSetAxesLimits.setLabel("Set Axes Limits ...");
          menuItemSetAxesLimits.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(java.awt.event.ActionEvent evt) {
                  menuItemSetAxesLimitsActionPerformed(evt);
              }
          }
          );
          menuAxis.add(menuItemSetAxesLimits);
          
        menuAxis.addSeparator();
          
        menuItemAxisCopy.setEnabled(false);
          menuItemAxisCopy.setLabel("Copy");
          menuAxis.add(menuItemAxisCopy);
          
        menuItemAxisPaste.setEnabled(false);
          menuItemAxisPaste.setLabel("Paste");
          menuAxis.add(menuItemAxisPaste);
          menuBar.add(menuAxis);
        
      menuLines.setName("menuLines");
        menuLines.setLabel("Lines");
        
        checkboxMenuItemLines.setState(true);
          checkboxMenuItemLines.setName("chkmenuitemLines");
          checkboxMenuItemLines.setLabel("Lines");
          checkboxMenuItemLines.addItemListener(new java.awt.event.ItemListener() {
              public void itemStateChanged(java.awt.event.ItemEvent evt) {
                  checkboxMenuItemLinesItemStateChanged(evt);
              }
          }
          );
          menuLines.add(checkboxMenuItemLines);
          
        checkboxMenuItemPoints.setName("chkmenuitemPoints");
          checkboxMenuItemPoints.setLabel("Points");
          checkboxMenuItemPoints.addItemListener(new java.awt.event.ItemListener() {
              public void itemStateChanged(java.awt.event.ItemEvent evt) {
                  checkboxMenuItemPointsItemStateChanged(evt);
              }
          }
          );
          menuLines.add(checkboxMenuItemPoints);
          menuBar.add(menuLines);
        setLayout(new java.awt.FlowLayout());
      addComponentListener(new java.awt.event.ComponentAdapter() {
          public void componentResized(java.awt.event.ComponentEvent evt) {
              formComponentResized(evt);
          }
      }
      );
      addWindowListener(new java.awt.event.WindowAdapter() {
          public void windowClosing(java.awt.event.WindowEvent evt) {
              exitForm(evt);
          }
      }
      );
      setMenuBar(menuBar);
      
  }//GEN-END:initComponents

  private void menuItemHelpActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuItemHelpActionPerformed
// Add your handling code here:
    //Open the help window and display help text.
    new HelpFrame("plotFrameHelp.text");
  }//GEN-LAST:event_menuItemHelpActionPerformed

  private void menuItemSetAxesLimitsActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuItemSetAxesLimitsActionPerformed
    //Get the axes ranges from the time series plot.
    double[] xRange = timeSeriesPlot.getXRange();
    double[] yRange = timeSeriesPlot.getYRange();
    //Ask for new axes ranges.
    PlotAxisDialog dialog = new PlotAxisDialog(this,true,xRange,yRange);
    dialog.show();
    xRange = dialog.getXRange();
    yRange = dialog.getYRange();
    //Set the new axes ranges for the plot.
    timeSeriesPlot.setXRange(xRange[0],xRange[1]);
    timeSeriesPlot.setYRange(yRange[0],yRange[1]);
    dialog.dispose();
    timeSeriesPlot.repaint();
  }//GEN-LAST:event_menuItemSetAxesLimitsActionPerformed

  private void menuItemAxisFillActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuItemAxisFillActionPerformed
    timeSeriesPlot.fillPlot();
  }//GEN-LAST:event_menuItemAxisFillActionPerformed

  private void formComponentResized (java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_formComponentResized
// Add your handling code here:
    Insets ins = this.getInsets();
    Rectangle bounds = this.getBounds();
    timeSeriesPlot.setBounds(ins.left,ins.top,bounds.width-ins.left-ins.right,bounds.height-ins.top-ins.bottom);
    timeSeriesPlot.validate();
  }//GEN-LAST:event_formComponentResized

  private void menuItemPrintActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuItemPrintActionPerformed

      _print();
  }//GEN-LAST:event_menuItemPrintActionPerformed

  private void checkboxMenuItemXAxisItemStateChanged (java.awt.event.ItemEvent evt) {//GEN-FIRST:event_checkboxMenuItemXAxisItemStateChanged
    //Get the state.
    boolean xLog = checkboxMenuItemXAxis.getState();
    //Set the scale.
    timeSeriesPlot.setXLog(xLog);
    //Redraw.
    timeSeriesPlot.plotData();
    timeSeriesPlot.repaint();
  }//GEN-LAST:event_checkboxMenuItemXAxisItemStateChanged

  private void checkboxMenuItemLinesItemStateChanged (java.awt.event.ItemEvent evt) {//GEN-FIRST:event_checkboxMenuItemLinesItemStateChanged
    //Get the state.
    boolean lines = checkboxMenuItemLines.getState();
    boolean points = checkboxMenuItemPoints.getState();
    //Points or lines must be set.
    if (!lines && !points)
    {
      points = true;
      checkboxMenuItemPoints.setState(points);
    }
    //Set the state.
    timeSeriesPlot.setLineState(lines);
    timeSeriesPlot.setPointState(points);
    //Redraw.
    timeSeriesPlot.plotData();
    timeSeriesPlot.repaint();
  }//GEN-LAST:event_checkboxMenuItemLinesItemStateChanged

  private void checkboxMenuItemPointsItemStateChanged (java.awt.event.ItemEvent evt) {//GEN-FIRST:event_checkboxMenuItemPointsItemStateChanged
    //Get the state.
    boolean lines = checkboxMenuItemLines.getState();
    boolean points = checkboxMenuItemPoints.getState();
    //Points or lines must be set.
    if (!lines && !points)
    {
      lines = true;
      checkboxMenuItemLines.setState(lines);
    }
    //Set the state.
    timeSeriesPlot.setLineState(lines);
    timeSeriesPlot.setPointState(points);
    //Redraw.
    timeSeriesPlot.plotData();
    timeSeriesPlot.repaint();
  }//GEN-LAST:event_checkboxMenuItemPointsItemStateChanged

  private void checkboxMenuItemYAxisItemStateChanged (java.awt.event.ItemEvent evt) {//GEN-FIRST:event_checkboxMenuItemYAxisItemStateChanged
    //Get the state.
    boolean yLog = checkboxMenuItemYAxis.getState();
    //Set the scale.
    timeSeriesPlot.setYLog(yLog);
    //Redraw.
    timeSeriesPlot.plotData();
    timeSeriesPlot.repaint();
  }//GEN-LAST:event_checkboxMenuItemYAxisItemStateChanged

  private void menuItemQuitActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuItemQuitActionPerformed
// Add your handling code here:
    dispose();
//    System.exit(0);
  }//GEN-LAST:event_menuItemQuitActionPerformed

  /** Exit the Application */
  private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
    dispose();
//    System.exit (0);
  }//GEN-LAST:event_exitForm

  /**
  * The main method is used for testing.
  *
  * @param args the command line arguments
  */
  public static void main (String args[]) {
    //Test code for stand alone application.
    //Plot window 1.
    Date[] l = {new Date(0L), new Date(1000L), new Date(2000L), new Date(3000L), new Date(4000L)};
    double[] d1 = {0.0,1.0,4.0,9.0,16.0};
    TimeSeries ts = new TimeSeries(l,d1);
    ts.setShortDescription("This is test 1");
    TimeSeriesPlot tsp = new TimeSeriesPlot(ts);
    PlotFrame pf1 = new PlotFrame(tsp);
    pf1.show();
    //Plot window 2.
    double[] d2 = {0.0,-1.0,-4.0,-9.0,-16.0};
    ts = new TimeSeries(l,d2);
    ts.setShortDescription("This is test 2");
    tsp = new TimeSeriesPlot(ts);
    PlotFrame pf2 = new PlotFrame(tsp);
    pf2.show();
  }


  // Variables declaration - do not modify//GEN-BEGIN:variables
  private java.awt.MenuBar menuBar;
  private java.awt.Menu menuFile;
  private java.awt.MenuItem menuItemPrint;
  private java.awt.MenuItem menuItemQuit;
  private java.awt.Menu menuAxis;
  protected java.awt.CheckboxMenuItem checkboxMenuItemXAxis;
  protected java.awt.CheckboxMenuItem checkboxMenuItemYAxis;
  private java.awt.MenuItem menuItemAxisFill;
  private java.awt.MenuItem menuItemSetAxesLimits;
  private java.awt.MenuItem menuItemAxisCopy;
  private java.awt.MenuItem menuItemAxisPaste;
  private java.awt.Menu menuLines;
  protected java.awt.CheckboxMenuItem checkboxMenuItemLines;
  protected java.awt.CheckboxMenuItem checkboxMenuItemPoints;
  // End of variables declaration//GEN-END:variables

}