#! /usr/bin/perl -w
###
# $Id: 02-D-PS.pl,v 1.2 2001/05/04 14:48:30 stamper Exp $
#
#  Purpose
#  
#   To search for system datasets by property.
#   
#  Arguments/Operator input
#  
#   Argument        Description
#   ---------------- -----------
#   $session        A session id
#   $user           The user making the request
#   $source         The source of the request
#   $timeStart      Start of interval that a dataset should intersect
#   $timeEnd        End of interval that a dataset should intersect
#   @region         Regions of space that a dataset should cover
#   @particle       Particle types that a dataset should include
#   $energyMin      Start of energy range that a dataset should cover
#   $energyMinUnits Units for ditto
#   $energyMax      End of energy range that a dataset should cover
#   $energyMaxUnits Units for ditto
#   
#  Returns (on standard output)
#  
#   status.code     0 if by endCGI (success), 1 if by abortCGI (failure)
#   status.message  An appropriate message
#   datasets        An array of dataset identifiers 
#  
#  External routines
#  
#   Sedat.pm       Several utility routines in module Sedat.
#   
#  Description
#  
#  Author	Richard Stamper
#
# $Log: 02-D-PS.pl,v $
# Revision 2.0  2009/10/21 10:29:00  dhc
# Added support for MySQL to run from ODI backend
#
# Revision 1.2  2001/05/04 14:48:30  stamper
# Set the LD_LIBRARY_PATH environment variable.
#
# Revision 1.1  2000/10/16 15:20:40  stamper
# Search SYSTEM datasets by property.
#
#
###
use CGI qw(:cgi);
use Sedat;

# $remoteuser, $session, $user and $source are not needed in the ODI implementation
#$remoteuser     = remote_user();
#$session        = param('session');
#$user           = param('user');
#$source         = param('source');
$timeStart      = param('time.start');
$timeEnd        = param('time.end');
@region         = param('region');
@particle       = param('particle');
$energyMin      = param('energy.min');
$energyMinUnits = param('energy.min.units');
$energyMax      = param('energy.max');
$energyMaxUnits = param('energy.max.units');

print "Content-type: text/plain\n\n";

# Check for valid connection details
# This is now replaced by checks on connections to the ODI database
#checkConnection($user,$remoteuser,$session,$source);

# Reduce the Region and Particle lists to strings of the first characters
$regionSet   = join('',map {$_ = uc(substr($_,0,1))} @region);
$particleSet = join('',map {$_ = uc(substr($_,0,1))} @particle);

# Convert the energy values to keV
$energyMin = $energyMin * ((uc($energyMinUnits) eq 'KEV') ? 1e3 : 1e6);
$energyMax = $energyMax * ((uc($energyMaxUnits) eq 'KEV') ? 1e3 : 1e6);

# Set the LD_LIBRARY_PATH environment variable to the value of the
# STPDFDYNAMIC variable (needed on Suns)
$ENV{LD_LIBRARY_PATH} = $ENV{STPDFDYNAMIC};

# Open a connection to the ODI database
$dbh = ODI_connect() or abortCGI('Error connecting to ODI database');

@lrd = ();
# Get the Region, Particle and Energy range details from the ODI database
# At the same time, store the ODI dataset name
# This replaces:
#@lrd = `./get_lrddata`;
my $sth = $dbh->prepare("SELECT sedat_name, UPPER(sedat_regions), UPPER(sedat_particles),
                         name FROM dataset WHERE sedat_name IS NOT NULL
                         AND sedat_emin<=" . $energyMax . " AND sedat_emax>=" . $energyMin);
$sth->execute() or abortCGI($dbh->errstr);
while (my $ref = $sth->fetchrow_arrayref()) {
  push @lrd, join(',', $ref->[0], $ref->[1], $ref->[2], $ref->[3]);
}
$sth->finish();

# Work through list of LRD catalogue details
# Checks on energy range are already performed
@names = ();
@ODI_names = ();
foreach (@lrd) {
    chomp;
    # Break out the components
    ($name,$region,$particle,$ODI_name) = split(/,/);
    # Skip if region or particle type don't match
    next if ($region !~ /[$regionSet]/);
    next if ($particle !~ /[$particleSet]/);
    # If we get here we have a match so remove trailing blanks
    # from the name and save it
    $name =~ s/\s*$//;
    push @names,$name;
    push @ODI_names, $ODI_name;
}

# Next work through list of datasets selected checking timestamp ranges
@datasets = ();
@bounds = ();
while ($#names >= 0) {
# Get the epoch range from the ODI database
  my $dname = shift(@ODI_names);

  $sth = $dbh->prepare("SELECT epoch, millisec FROM " . $dname .
                       " ORDER BY cdf_epoch ASC LIMIT 1");
  $sth->execute() or abortCGI($dbh->errstr);
  $ref = $sth->fetchrow_arrayref();
  $sth->finish();
  my $tmin = substr($ref->[0], 0, 10) . 'T' . substr($ref->[0], 11, 8) . '.' . $ref->[1] . 'Z';

  $sth = $dbh->prepare("SELECT epoch, millisec FROM " . $dname .
                       " ORDER BY cdf_epoch DESC LIMIT 1");
  $sth->execute() or abortCGI($dbh->errstr);
  $ref = $sth->fetchrow_arrayref();
  $sth->finish();
  my $tmax = substr($ref->[0], 0, 10) . 'T' . substr($ref->[0], 11, 8) . '.' . $ref->[1] . 'Z';

  push @bounds, join(',', shift(@names), $tmin, $tmax);
}

# The results lines have 'name,start-timestamp,end-timestamp'
foreach (@bounds) {
  chomp;
  ($name,$lower,$upper) = split /,/;
# Save if the time range overlaps that requested
  if (($lower le $timeEnd) && ($upper ge $timeStart)) {
    push @datasets,$name;
  }
}

# Close the connection to the ODI database
$dbh->disconnect();

# Return the dataset details and listing
returnField('datasets','T',@datasets);
endCGI('Matching system dataset names returned');
