PRO odi_cdf, cdf_name, dataset_name, varnames=varnames, $
             epochrange=epochrange, recordrange=recordrange

  dataset_name = STRSPLIT(dataset_name, 'SYSTEM!', /EXTRACT, /REGEX)

  IF (STRPOS(cdf_name, '.cdf') LT 0) THEN cdf_name += '.cdf'
  FILE_DELETE, cdf_name, /ALLOW_NONEXISTENT

;
; Construct the epoch or time range in SQL
;
  SQL_range, epochrange=epochrange, recordrange=recordrange, sqlrange

; Connect to the database
  DBConnect, oDB

; Find the dataset id
  oRS = OBJ_NEW('IDLDBRecordSet', oDB, $
                SQL="SELECT id, name, LEFT(majority, 20) AS majority " + $
                    "FROM dataset WHERE " + $
                    "sedat_name='" + STRUPCASE(dataset_name) + "'")
  IF (oRS->CurrentRecord() GE 0) THEN BEGIN
    ds = oRS->GetRecord()
    OBJ_DESTROY, oRS
  ENDIF ELSE $
    error_exit, ' Dataset ' + dataset_name + ' not found!'

  cdf_id = CDF_CREATE(cdf_name, /CLOBBER, /SINGLE_FILE, /NETWORK_ENCODING, $
                      ROW_MAJOR=FIX(ds.majority EQ 'ROW'), $
                      COL_MAJOR=FIX(ds.majority NE 'ROW'))
;
; Retrieve and store global attributes from table dataset_metadata
;
  sql = "SELECT LEFT(dm.attribute, 100) AS name, " + $
        "LEFT(dt.idl_type, 20) AS type, dm.entry_number, " + $
        "LEFT(dm.value, 1000) AS value FROM dataset_metadata AS dm " + $
        "LEFT JOIN data_type AS dt ON (dm.data_type_id=dt.id) " + $
        "WHERE dm.dataset_id=" + STRING(ds.id) + " AND " + $
        "dm.attribute!='platform' AND dm.attribute!='platform_type' " + $
        "AND dm.attribute!='instrument'"
  oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=sql)
  IF (oRS->CurrentRecord() GE 0) THEN BEGIN
    att = oRS->GetRecord()
    WHILE (oRS->MoveCursor(/NEXT)) DO $
      att = [att, oRS->GetRecord()]
    OBJ_DESTROY, oRS
  ENDIF ELSE $
    error_exit, ' Error reading global attributes!'

  FOR i=0,N_ELEMENTS(att)-1 DO BEGIN
    IF (att[i].entry_number EQ 1) THEN $
      att_id = CDF_ATTCREATE(cdf_id, att[i].name, /GLOBAL_SCOPE)
    IF (att[i].name EQ 'Logical_file_id') THEN BEGIN
      aval = cast_odi_idl(cdf_name, att[i].type)
    ENDIF ELSE IF (att[i].name EQ 'Generated_by') THEN BEGIN
      IF (!VERSION.OS_FAMILY EQ 'Windows') THEN $
        hostname = GETENV('Computername') $
      ELSE $
        hostname = GETENV('HOSTNAME')
      aval = cast_odi_idl('ODI V1.0 on ' + hostname, att[i].type)
    ENDIF ELSE IF (att[i].name EQ 'Generation_date') THEN BEGIN
      aval = cast_odi_idl(SYSTIME(/UTC), att[i].type)
    ENDIF ELSE BEGIN
      aval = cast_odi_idl(att[i].value, att[i].type)
    ENDELSE
    CDF_ATTPUT, cdf_id, att_id, att[i].entry_number-1, aval
  ENDFOR

;
; Retrieve and store list of variable attributes from table variable_attribute
;
  sql = "SELECT LEFT(va.name, 100) AS name FROM " + $
        "variable_attribute AS va LEFT JOIN variable_metadata AS vm " + $
        "ON (va.variable_metadata_id=vm.id) " + $
        "WHERE vm.dataset_id=" + STRING(ds.id) + " GROUP BY va.name " + $
        "ORDER BY va.name"
  oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=sql)
  IF (oRS->CurrentRecord() GE 0) THEN BEGIN
    att = oRS->GetRecord()
    WHILE (oRS->MoveCursor(/NEXT)) DO $
      att = [att, oRS->GetRecord()]
    OBJ_DESTROY, oRS
  ENDIF ELSE $
    error_exit, ' Error reading variable attributes!'

  FOR i=0,N_ELEMENTS(att)-1 DO $
    att_id = CDF_ATTCREATE(cdf_id, att[i].name, /VARIABLE_SCOPE)

;
; Retrieve and store variable definitions from table variable_metadata
;
  IF (N_ELEMENTS(varnames) EQ 0) THEN varnames = '*'

  var = odi_vardefs(varnames, ds.id, oDB)

;
; Store variables
; Retrieve and store variable attributes from table variable_attribute
;
  FOR i=0,N_ELEMENTS(var)-1 DO BEGIN
    id = create_cdf_var(cdf_id, var[i])
    sql = "SELECT va.id, LEFT(va.name, 50) AS name, " + $
          "LEFT(va.value, 500) AS value, " + $
          "LEFT(dt.cdf_type, 20) AS cdf_type, " + $
          "LEFT(dt.idl_type, 20) AS idl_type FROM " + $
          "variable_attribute AS va LEFT JOIN data_type AS dt " + $
          "ON (va.data_type_id=dt.id) " + $
          "WHERE va.variable_metadata_id=" + STRING(var[i].id)
    oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=sql)
    IF (oRS->CurrentRecord() GE 0) THEN BEGIN
      att = oRS->GetRecord()
      WHILE (oRS->MoveCursor(/NEXT)) DO $
        att = [att, oRS->GetRecord()]
      OBJ_DESTROY, oRS
    ENDIF ELSE $
      error_exit, ' Error reading variable attributes for ' + var[i].name + '!'
    FOR j=0,N_ELEMENTS(att)-1 DO BEGIN
      IF (att[j].cdf_type EQ 'CDF_EPOCH') THEN BEGIN
        CDF_ATTPUT, cdf_id, att[j].name, var[i].name, $
                    cdf_date2cdf_epoch(att[j].value)
      ENDIF ELSE BEGIN
        CDF_ATTPUT, cdf_id, att[j].name, var[i].name, $
                    cast_odi_idl(att[j].value, att[j].idl_type)
      ENDELSE
    ENDFOR
;
; For variables with record variance FALSE, write NRV values
;
    IF (var[i].rvar EQ 'F') THEN BEGIN
      sql = "SELECT LEFT(vn.value, 1000) AS value FROM variable_nrv AS vn " + $
            "WHERE vn.variable_metadata_id=" + STRING(var[i].id) + $
            " ORDER BY ind ASC"
      oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=sql)
      IF (oRS->CurrentRecord() GE 0) THEN BEGIN
        nrv = oRS->GetRecord()
        WHILE (oRS->MoveCursor(/NEXT)) DO $
          nrv = [nrv, oRS->GetRecord()]
        OBJ_DESTROY, oRS
      ENDIF ELSE $
        error_exit, ' Error reading RV values for ' + var[i].name + '!'
      IF (var[i].dims EQ 0) THEN BEGIN
        dnrv = STRARR(1)
      ENDIF ELSE BEGIN
        res = EXECUTE("dnrv = STRARR(" + $
                      STRJOIN(REVERSE(STRSPLIT(var[i].sizes, ',', /EXTRACT)), ',') + $
                      ")")
      ENDELSE
      nr = SIZE(dnrv)
      nr = nr[N_ELEMENTS(nr)-1]
      IF (nr NE N_ELEMENTS(nrv)) THEN $
        error_exit, ' Size of variable ' + var[i].name + $
                    ' does not match number of NRV values!'
;      FOR k=0,nr-1 DO
      dnrv[*] = nrv.value
;      IF (ds.majority EQ 'ROW') THEN dnrv = TRANSPOSE(dnrv)
      CDF_VARPUT, cdf_id, var[i].name, cast_odi_idl(dnrv, var[i].idl_type)
    ENDIF
  ENDFOR


;
; Read the record varying variables from the database and
; write them to the cdf file
;

; Filter out the record varying variables
  ind = WHERE(var.rvar EQ 'T', count)
  IF (count GT 0) THEN BEGIN
    var = var[ind]

    odi_readvars, var, ds.majority, vardesc, sql

    sql += " FROM " + ds.name + sqlrange

    oRS = OBJ_NEW('IDLDBRecordSet', oDB, N_BUFFERS=10000, SQL=sql)

    nrec = 0L
    IF (oRS->CurrentRecord() GE 0) THEN status = 1 ELSE status = -1
    WHILE (status GT 0) DO BEGIN
      data = oRS->GetRecord()
      nr = 0
      FOR i=0,N_ELEMENTS(var)-1 DO BEGIN
        res = EXECUTE("dvar = " + vardesc[i].vtype)
        dvar = cast_odi_idl(dvar, var[i].idl_type)
        IF (var[i].dims GT 1) THEN dvar = TRANSPOSE(dvar)
        nd = SIZE(dvar)
        nd = nd[N_ELEMENTS(nd)-1]
        FOR j=0,nd-1 DO dvar[j] = data.(nr+j)
        CDF_VARPUT, cdf_id, var[i].name, dvar, REC_START=nrec
        nr += nd
      ENDFOR
      nrec += 1
      status = oRS->MoveCursor(/NEXT)
    ENDWHILE
    OBJ_DESTROY, oRS
  ENDIF

  CDF_CLOSE, cdf_id

  OBJ_DESTROY, oDB

END