FUNCTION cdf_date2cdf_epoch, date

  epoch = -1.0D0

  months = ['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec']

  ddate = STRSPLIT(date, /EXTRACT)
  cdate = STRSPLIT(ddate[0], '-', /EXTRACT)
  day = FIX(cdate[0])
  month = WHERE(months EQ cdate[1])
  month = month[0] + 1
  year = FIX(cdate[2])
  ctime = STRSPLIT(ddate[1], ':', /EXTRACT)
  hour = FIX(ctime[0])
  minute = FIX(ctime[1])
  secs = STRSPLIT(ctime[2], '.', /EXTRACT)
  second = FIX(secs[0])
  IF (N_ELEMENTS(secs) GT 1) THEN msecs = FIX(secs[1]) ELSE msecs = 0

  cdf_epoch, epoch, year, month, day, hour, minute, second, msecs, /COMPUTE_EPOCH

  return, epoch

END


FUNCTION sql_date2cdf_epoch, date

  epoch = -1.0D0

  cdf_epoch, epoch, FIX(STRMID(date, 0, 4)), FIX(STRMID(date, 5, 2)), $
             FIX(STRMID(date, 8, 2)), FIX(STRMID(date, 11, 2)), $
             FIX(STRMID(date, 14, 2)), FIX(STRMID(date, 17, 2)), $
             FIX(STRMID(date, 20, 3)), /COMPUTE_EPOCH

  return, epoch

END


FUNCTION create_cdf_var, cdf_id, var

  stmt = "id = CDF_VARCREATE(cdf_id, '" + var.name  + "'"

  IF (var.dims EQ 0) THEN BEGIN
    stmt += ", ['VARY']"
  ENDIF ELSE BEGIN
    stmt += ", ["
    rvars = STRSPLIT(var.dvar, ",", /EXTRACT)
    FOR i=0,var.dims-1 DO BEGIN
      IF (i GT 0) THEN stmt += ","
      IF (rvars[i] EQ 'T') THEN $
        stmt += "'VARY'" $
      ELSE $
        stmt += "'NOVARY'"
    ENDFOR
    stmt += "], DIMENSIONS=[" + $
            STRJOIN(REVERSE(STRSPLIT(var.sizes, ',', /EXTRACT)), ',') + "]"
  ENDELSE

  stmt += ", /" + var.cdf_type + $
          ", NUMELEM=" + STRING(var.number_elements) + $
          ", REC_NOVARY=" + STRING(FIX(var.rvar EQ 'F'), FORMAT='(I1)') + $
          ", REC_VARY=" + STRING(FIX(var.rvar EQ 'T'), FORMAT='(I1)') + $
          ", /ZVARIABLE)"

  res = EXECUTE(stmt)

  IF (res EQ 0) THEN error_exit, ' Error creating cdf variable ' + var.name

  return, id

END


FUNCTION cast_odi_sedat, var, type

  IF (type EQ 'C') THEN cvar = STRING(var) $
  ELSE IF (type EQ 'I') THEN cvar = LONG(var) $
  ELSE IF (type EQ 'R') THEN cvar = FLOAT(var) $
  ELSE IF (type EQ 'D') THEN cvar = DOUBLE(var) $
  ELSE cvar = 'error'

  RETURN, cvar

END


FUNCTION cast_odi_idl, invar, type

  IF (type EQ 'BYTE') THEN BEGIN
    outvar = BYTE(invar)
  ENDIF ELSE IF (type EQ 'INT') THEN BEGIN
    outvar = FIX(invar)
  ENDIF ELSE IF (type EQ 'UINT') THEN BEGIN
    outvar = UINT(invar)
  ENDIF ELSE IF (type EQ 'LONG') THEN BEGIN
    outvar = LONG(invar)
  ENDIF ELSE IF (type EQ 'ULONG') THEN BEGIN
    outvar = ULONG(invar)
  ENDIF ELSE IF (type EQ 'FLOAT') THEN BEGIN
    outvar = FLOAT(invar)
  ENDIF ELSE IF (type EQ 'DOUBLE') THEN BEGIN
    outvar = DOUBLE(invar)
  ENDIF ELSE IF (type EQ 'STRING') THEN BEGIN
    outvar = STRING(invar)
  ENDIF ELSE BEGIN
    error_exit, ' Variable type ' + type + ' is not a valid type!'
  ENDELSE

  return, outvar

END


PRO error_exit, message

  PRINT,'*****'
  PRINT,'*****'
  PRINT,'*****'
  PRINT,'*****', message
  PRINT,'*****'
  PRINT,'*****'
  PRINT,'*****'

  EXIT, STATUS=1

END


PRO DBConnect, oDB

  status = DB_EXISTS()

  IF (~status) THEN error_exit, " Error starting IDL DataMiner"

  oDB = OBJ_NEW('IDLDBDatabase')

  ds = "ODI_SEDAT"
  sources = oDB->GetDataSources()
  dbindex = WHERE(sources.datasource EQ ds, n)
  IF (n LT 1) THEN error_exit, " Error connecting to " + ds

  oDB->Connect, DataSource=ds

END


;
; Construct the epoch or time range in SQL
;
PRO SQL_range, epochrange=epochrange, recordrange=recordrange, sql

  IF ((N_ELEMENTS(epochrange) EQ 0) AND (N_ELEMENTS(recordrange) EQ 0)) THEN BEGIN
    sql = ' ORDER BY cdf_epoch ASC'
  ENDIF ELSE IF ((N_ELEMENTS(epochrange) GT 0) AND (N_ELEMENTS(recordrange) GT 0)) THEN BEGIN
    error_exit, ' Epoch range and record range cannot both be specified!'
  ENDIF ELSE IF (N_ELEMENTS(epochrange) GT 0) THEN BEGIN
    e1 = sql_date2cdf_epoch(epochrange[0])
    IF (e1 LT 0.0D0) THEN $
      error_exit, ' Invalid epoch: ' + epochrange[0]
    e2 = sql_date2cdf_epoch(epochrange[1])
    IF (e2 LT 0.0D0) THEN $
      error_exit, ' Invalid epoch: ' + epochrange[1]
    IF (e1 GT e2) THEN $
      error_exit, ' Last epoch should be greater than first!'
    sql = ' WHERE cdf_epoch>=' + STRING(e1) + ' AND cdf_epoch <=' + STRING(e2) + $
          ' ORDER BY cdf_epoch ASC'
  ENDIF ELSE BEGIN
    IF (recordrange[0] GT recordrange[1]) THEN $
      error_exit, ' Last record number should be greater than first!'
    sql = ' ORDER BY cdf_epoch ASC LIMIT ' + STRING(recordrange[0]-1) + "," + $
          STRING(recordrange[1]-recordrange[0]+1)
  ENDELSE

END


;
; Retrieve and store variable definitions from table variable_metadata
;
FUNCTION odi_vardefs, varnames, dataset_id, oDB

  sql = "SELECT vm.id, LEFT(vm.name, 100) AS name, vm.number_elements, " + $
        "vm.dims, LEFT(vm.sizes, 500) AS sizes, " + $
        "LEFT(vm.record_variance, 10) AS rvar, " + $
        "LEFT(vm.dimension_variances, 100) AS dvar, " + $
        "LEFT(dt.cdf_type, 20) AS cdf_type, " + $
        "LEFT(dt.sedat_type, 20) AS sedat_type, " + $
        "LEFT(dt.idl_type, 20) AS idl_type FROM " + $
        "variable_metadata AS vm LEFT JOIN data_type AS dt " + $
        "ON (vm.data_type_id=dt.id) " + $
        "WHERE vm.dataset_id=" + STRING(dataset_id)
  IF (varnames[0] NE '*') THEN BEGIN
    sql += " AND (vm.name='" + varnames[0] + "'"
    FOR i=1,N_ELEMENTS(varnames)-1 DO $
      sql += " OR vm.name='" + varnames[i] + "'"
    sql += ")"
  ENDIF
  oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=sql)
  IF (oRS->CurrentRecord() GE 0) THEN BEGIN
    var = oRS->GetRecord()
    WHILE (oRS->MoveCursor(/NEXT)) DO $
      var = [var, oRS->GetRecord()]
    OBJ_DESTROY, oRS
  ENDIF ELSE $
    error_exit, ' Error reading variable definitions!'
  IF ((varnames[0] NE '*') AND $
      (N_ELEMENTS(varnames) NE N_ELEMENTS(var))) THEN $
    error_exit, ' Not all variables were found!'

  RETURN, var

END


PRO odi_readvars, var, dataset_majority, vardesc, sql

    nvar = N_ELEMENTS(var)
    vardesc = {vard, dtype: '', vtype: '', sql: '', ndims: 0L, dimensions: LONARR(10)}
    vardesc = REPLICATE(vardesc, nvar)
    FOR i=0,nvar-1 DO BEGIN
      IF (var[i].dims EQ 0) THEN BEGIN
        vardesc[i].vtype = 'STRARR(1)'
        IF (STRUPCASE(var[i].name) EQ 'EPOCH') THEN BEGIN
          vardesc[i].sql = 'cdf_epoch AS Epoch'
        ENDIF ELSE BEGIN
          sqlvar = STRLOWCASE(var[i].name)
          IF (var[i].idl_type EQ 'STRING') THEN BEGIN
            vardesc[i].sql = 'LEFT(' + sqlvar + ', 1000) AS ' + sqlvar
          ENDIF ELSE BEGIN
            vardesc[i].sql = sqlvar
          ENDELSE
        ENDELSE
      ENDIF ELSE IF (var[i].dims EQ 1) THEN BEGIN
        vardesc[i].vtype = 'STRARR(' + var[i].sizes + ')'
        vardesc[i].sql = ''
        FOR j=0,var[i].sizes-1 DO BEGIN
          IF (j GT 0) THEN vardesc[i].sql += ','
          sqlvar = STRLOWCASE(var[i].name) + '_' + STRTRIM(STRING(j+1), 2)
          IF (var[i].idl_type EQ 'STRING') THEN BEGIN
            vardesc[i].sql += 'LEFT(' + sqlvar + ', 1000) AS ' + sqlvar
          ENDIF ELSE BEGIN
            vardesc[i].sql += sqlvar
          ENDELSE
        ENDFOR
      ENDIF ELSE IF (var[i].dims EQ 2) THEN BEGIN
        vardesc[i].vtype = 'STRARR(' + $
                           STRJOIN(STRSPLIT(var[i].sizes, ',', /EXTRACT), ',') + ')'
        IF (STRUPCASE(STRMID(dataset_majority, 0, 3)) EQ 'ROW') THEN $
          sizes = STRSPLIT(var[i].sizes, ',', /EXTRACT) $
        ELSE $
          sizes = REVERSE(STRSPLIT(var[i].sizes, ',', /EXTRACT))
        vardesc[i].sql = ''
        FOR s1=1,sizes[0] DO BEGIN
          FOR s2=1,sizes[1] DO BEGIN
            IF (s1+s2 GT 2) THEN vardesc[i].sql += ','
            sqlvar = STRLOWCASE(var[i].name) + '_' + $
                     STRTRIM(STRING(s1), 2) + '_' + $
                     STRTRIM(STRING(s2), 2)
            IF (var[i].idl_type EQ 'STRING') THEN BEGIN
              vardesc[i].sql += 'LEFT(' + sqlvar + ', 1000) AS ' + sqlvar
            ENDIF ELSE BEGIN
              vardesc[i].sql += sqlvar
            ENDELSE
          ENDFOR
        ENDFOR
      ENDIF ELSE BEGIN
        error_exit, 'Only variables with up to 2 dimensions are supported in this version'
      ENDELSE
    ENDFOR
    sql = "SELECT "
    FOR i=0,nvar-1 DO BEGIN
      IF (i GT 0) THEN sql += ","
      sql += vardesc[i].sql
    ENDFOR

END