
PRO db_channels

  db_lib

  Namelist = read_nml('idl')

  if (isDefined('PRIMARY',Namelist)) then begin
    pname = str_sep(NMLValue('PRIMARY',Namelist),'''')
    pname = pname[1]
  endif else $
    errorMessage,'Primary dataset not defined'

  DBConnect, oDB

  OPENW, 1, "dbtemp.txt"

  oRS = OBJ_NEW('IDLDBRecordSet', oDB, $
                SQL="SELECT ds.id, ds.name, " + $
                    "ds.epoch_start AS tmin, ds.epoch_end AS tmax, " + $
                    "LEFT(dm.value, 500) AS d " + $
                    "FROM dataset AS ds LEFT JOIN dataset_metadata AS dm " + $
                    "ON (ds.id=dm.dataset_id) WHERE ds.name='" + pname + $
                    "' AND dm.attribute='Logical_source_description'")
  primary = oRS->GetRecord()
  OBJ_DESTROY, oRS

  cdfepoch_to_string, primary.tmin, tmin
  cdfepoch_to_string, primary.tmax, tmax

  PRINTF, 1, "<H4>" + primary.d + " [" + tmin + " - " + tmax + "]</H4>"

  PRINTF, 1, "<TABLE border='0' cellspacing='2' celltabbing='4'>"

  query = "SELECT vm.id, LEFT(vm.name, 500) AS name, vm.dims, " + $
          "LEFT(vm.sizes, 500) AS sizes " + $
          "FROM variable_metadata AS vm " + $
          "WHERE UPPER(vm.name)!='EPOCH' AND " + $
          "vm.dataset_id=" + STRING(primary.id) + $
          " AND vm.record_variance='T'";
  oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=query)
  vars = oRS->GetRecord()
  WHILE (oRS->MoveCursor(/NEXT)) DO $
    vars = [vars, oRS->GetRecord()]
  OBJ_DESTROY, oRS

  FOR i=0,N_ELEMENTS(vars)-1 DO BEGIN
    query = "SELECT LEFT(value, 500) AS value FROM variable_attribute " + $
            "WHERE variable_metadata_id=" + STRING(vars[i].id) + $
            " AND (UPPER(LEFT(name, 8))='FIELDNAM' OR " + $
            "UPPER(name)='UNITS') AND CHAR_LENGTH(TRIM(value))>0 " + $
            "ORDER BY name"
    oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=query)
    rec = oRS->GetRecord()
    fname = rec.value
    funits = ""
    status = oRS->MoveCursor(/NEXT)
    IF (status EQ 1) THEN BEGIN
      rec = oRS->GetRecord()
      funits = rec.value
    ENDIF
    OBJ_DESTROY, oRS

    IF (vars[i].dims LE 0) THEN BEGIN
      PRINTF, 1, "<TR><TD align='left'><SELECT name='" + $
                 STRLOWCASE(vars[i].name) + "_0' datatype='integer'>"
      PRINTF, 1, "<OPTION value='0'>Not selected</OPTION>"
      PRINTF, 1, "<OPTION value='1'>Mean</OPTION>"
      PRINTF, 1, "</SELECT></TD><TD align='left'>" + fname
      IF (STRLEN(STRCOMPRESS(funits, /R)) GT 0) THEN $
        PRINTF, 1, " [" + funits + "]"
      PRINTF, 1, "</TD></TR>"
    ENDIF ELSE BEGIN
;
; Selection as spectrogram
;
      PRINTF, 1, "<TR><TD align='left'><SELECT name='" + $
                 STRLOWCASE(vars[i].name) + "' datatype='integer'>"
      PRINTF, 1, "<OPTION value='0'>Not selected</OPTION>"
      PRINTF, 1, "<OPTION value='1'>Mean</OPTION>"
      PRINTF, 1, "</SELECT></TD><TD align='left'>" + fname
      IF (STRLEN(STRCOMPRESS(funits, /R)) GT 0) THEN $
        PRINTF, 1, " [" + funits + "]"
      PRINTF, 1, " as spectrogram</TD></TR>"
;
; Individual channels
;
      PRINTF, 1, "<TR><TD align='left'><SELECT name='" + $
                 STRLOWCASE(vars[i].name) + "_channels' datatype='integer'>"
      PRINTF, 1, "<OPTION value='0'>Not selected</OPTION>"
      PRINTF, 1, "<OPTION value='1'>Mean</OPTION>"
      PRINTF, 1, "</SELECT></TD><TD align='left'>" + fname
      IF (STRLEN(STRCOMPRESS(funits, /R)) GT 0) THEN $
        PRINTF, 1, " [" + funits + "] as individual channels"
      PRINTF, 1, "</TD></TR>"

      query = "SELECT vm.id AS id FROM variable_metadata AS vm " + $
              "LEFT JOIN variable_attribute AS va " + $
              "ON (vm.name=va.value) WHERE va.name='LABL_PTR_1' AND " + $
              "va.variable_metadata_id=" + STRING(vars[i].id)
      oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=query)
      rec = oRS->GetRecord()
      depid = rec.id
      OBJ_DESTROY, oRS

      query = "SELECT ind, LEFT(value, 500) AS value " + $
              "FROM variable_nrv WHERE variable_metadata_id= " + $
              STRING(depid)
      oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=query)
      nrv = oRS->GetRecord()
      FOR j=1,FIX(vars[i].sizes)-1 DO BEGIN
        status = oRS->MoveCursor(/NEXT)
        nrv = [nrv, oRS->GetRecord()]
      ENDFOR
      OBJ_DESTROY, oRS
      nrv = nrv[SORT(nrv.ind)]

      query = "SELECT LEFT(value, 500) AS value FROM variable_attribute WHERE " + $
              "variable_metadata_id=" + STRING(depid) + $
              " AND UPPER(name)='UNITS' AND " + $
              "CHAR_LENGTH(TRIM(value))>0"
      oRS = OBJ_NEW('IDLDBRecordSet', oDB, SQL=query)
      status = oRS->MoveCursor(/FIRST)
      nrv_units = ""
      IF (status EQ 1) THEN BEGIN
        rec = oRS->GetRecord()
        nrv_units = rec.value
      ENDIF
      OBJ_DESTROY, oRS

      FOR j=0,vars[i].sizes-1 DO BEGIN
        PRINTF, 1, "<TR><TD>&nbsp;</TD>"
        PRINTF, 1, "<TD align='left'>" + $
                   "<INPUT type='checkbox' name='" + $
                   STRLOWCASE(vars[i].name) + "_" + $
                   STRCOMPRESS(STRING(j+1), /R) + "' value='Y'>&nbsp;"
        PRINTF, 1, nrv[j].value
        IF (STRLEN(STRCOMPRESS(nrv_units, /R)) GT 0) THEN $
          PRINTF, 1, " " + nrv_units
        PRINTF, 1, "</TD></TR>"
      ENDFOR
    ENDELSE
  ENDFOR

  PRINTF, 1, "</TABLE><P>"

  CLOSE, 1

  OBJ_DESTROY, oDB

END