<?php
/**
* Parser function for Dst.
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 2 December, 2009 
*
* @version $Id: Dst.php 224 2009-12-03 12:49:21Z peter $
*
* @package ODI
* @subpackage ParsersFunctions
*/


  function dir_Dst($basedir)
  {
// Scan files in data directory
    $dfiles = dirscan($basedir, "dst%");

// Add the Kyoto HTML pages and Lund forecast page to the file list for Dst
    $dbasedir = "http://swdc234.kugi.kyoto-u.ac.jp/dst_realtime";
    $findex = file($dbasedir . "/index.html");
    if ($findex !== FALSE)
    {
      foreach ($findex as $dumstr)
      {
        if (($di = strpos($dumstr, "/index.html")) &&
            (strpos($dumstr, "month") === FALSE) &&
            (strpos($dumstr, "dst") === FALSE))
        {
          $file = $dbasedir . substr($dumstr, $di-7, 18);
          $dum = file($file);
          foreach ($dum as $line)
          {
            $offset = strpos($line, "[Updated");
            if ($offset !== FALSE)
            {
              $date = explode("UT", substr($line, $offset+11));
              $date = date("Y/m/d H:i:s", strtotime($date[0] . ":00"));
              break;
            }
          }
          $dfiles[] = array($file, $date, $file);
        }
      }
    }
    $dfiles[] = array("http://rwc.lund.irf.se/rwc/dst/dst1.txt",
                      date("Y/m/d H:i:s"),
                      "http://rwc.lund.irf.se/rwc/dst/dst1.txt");

    return $dfiles;
  }


// Parse a data file and write formatted content to load.tmp
  function parse_Dst($rname, $fid)
  {
    $ms = 0;
    $dsts = file($rname);
    $tfile = fopen("load.tmp", "w");
    if ($dsts !== FALSE)
    {
      if (strpos($rname, "lund") !== FALSE)
      {
        $dum = preg_split("/\s+/", trim($dsts[0]));
        $date = $dum[0];
        $hour = substr($dum[1], 0, 2);
        $epoch = $date . " " . $hour . ":00:00";
        $dstval = floatval($dum[2]);
        $cdf_epoch = date_to_cdfepoch($epoch, $ms);
        fprintf($tfile, "%20.3f, %s, %d, %d,% d, %d%s",
                        $cdf_epoch, $epoch, $ms, $dstval, 3, $fid, LF);
      }
      else if (strpos($rname, "kyoto") !== FALSE)
      {
        $di = strpos($rname, "/index.html");
        $year = substr($rname, $di-6, 4);
        $month = substr($rname, $di-2, 2);
        $ndays = intval(strftime("%e", mktime(0, 0, 0, $month+1, 0, $year)));
        $nq = intval(floor($ndays/5));
        $nr = $ndays - $nq * 5;
        $dumstr = array_shift($dsts);
        while (strpos($dumstr, "DAY") === FALSE)
          $dumstr = array_shift($dsts);
        for ($i = 0; $i < $nq*6+$nr; $i++)
        {
          $dumstr = array_shift($dsts);
          if (($i+1) % 6 > 0)
          {
            $day = sprintf("%02d", substr($dumstr, 0, 2));
            for ($j = 0; $j < 3; $j++)
            {
              for ($k = 0; $k < 8; $k++)
              {
                $epoch = $year . "-" . $month . "-" . $day . " " .
                         sprintf("%02d", $j*8+$k) . ":00:00";
                $cdf_epoch = date_to_cdfepoch($epoch, $ms);
                $dstval = intval(substr($dumstr, 3+$j*33+$k*4, 4));
//                if ($dstval == 9999) $dstval = "NULL";
                fprintf($tfile, "%20.3f, %s, %d,%s, %d, %d%s",
                                $cdf_epoch, $epoch, $ms, $dstval, 2, $fid, LF);
              }
            }
          }
        }
      }
      else
      {
        foreach ($dsts as $dst)
        {
          $year = intval(substr($dst, 3, 2));
          if ($year > 30) $year += 1900; else $year += 2000;
          $month = intval(substr($dst, 5, 2));
          if ($month > 0)
          {
            $day = intval(substr($dst, 8, 2));
            $flag = substr($dst, 7, 1);
            if ($flag == "*") $flag = 0; else $flag = 1;
            for ($i = 0; $i < 24; $i++)
            {
              $dstval = intval(substr($dst, 20+$i*4, 4));
              $epoch = sprintf("%04d-%02d-%02d %02d:00:00", $year, $month, $day, $i);
              $cdf_epoch = date_to_cdfepoch($epoch, $ms);
              fprintf($tfile, "%20.3f, %s, %d, %d, %d, %d%s",
                              $cdf_epoch, $epoch, $ms, $dstval, $flag, $fid, LF);
            }
          }
        }
      }
      fclose($tfile);
      $result = TRUE;
    }
    else
      $result = FALSE;

    return $result;
  }

?>