<?php
/**
* Parser function for GOES SEM.
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 2 December, 2009 
*
* @version $Id: GOES_SEM.php 224 2009-12-03 12:49:21Z peter $
*
* @package ODI
* @subpackage ParsersFunctions
*/


// Parse a data file and write formatted content to load.tmp
  function parse_GOES_SEM($rname, $fid)
  {
// Read in GOES longitude file
    $longs = file("http://goes.ngdc.noaa.gov/docs/GoesLongitudes.txt");
    if ($longs === FALSE) return FALSE;
    while (strpos($longs[0], "------") === FALSE) array_shift($longs);
    array_shift($longs);

    $gnr = substr($rname, -11, 2);
    $yymm = substr($rname, -8, 4);
    $glon = array();
    for ($i = 0; $i < 31; $i++) $glon[] = "0.0";
    foreach ($longs as $long)
    {
      if (substr($long, 5, 7) == $gnr . " " . $yymm)
      {
        $glon = preg_split("/[\s]+/", trim(substr($long, 12)));
        break;
      }
    }

    $ms = 0;
    $goes = file($rname);
    if ($goes !== FALSE)
    {
      $efile = fopen("epoch.tmp", "w");
      $vfile = fopen("values.tmp", "w");
      while (strpos($goes[0], "------") === FALSE) array_shift($goes);
      array_shift($goes);
      foreach ($goes as $sdum)
      {
        list($year, $month, $day, $hour, $min) = sscanf($sdum, "%2d%2s%2s %2s%2s");
        if ($year < 50) $year += 2000; else $year += 1900;
        $epoch = sprintf("%04d-%02d-%02d %02d:%02d:00", $year, $month, $day, $hour, $min);
        $cdf_epoch = date_to_cdfepoch($epoch, $ms);

        $xrays = implode(",", preg_split("/[\s]+/", trim(substr($sdum, 15, 20))));
        $magc = implode(",", preg_split("/[\s]+/", trim(substr($sdum, 35, 27))));
        $mags = trim(substr($sdum, 62, 9));
        $el = trim(substr($sdum, 71, 10));
        $pr = implode(",", preg_split("/[\s]+/", trim(substr($sdum, 81))));

        fprintf($vfile, "%s,%s,%s,%s,%s, %d%s",
                        $xrays, $magc, $mags, $el, $pr, $fid, LF);
        fprintf($efile, "%20.3f, %s, %3d, %7.2f%s",
                        $cdf_epoch, $epoch, $ms, $glon[$day-1], LF);
      }
      fclose($vfile);
      fclose($efile);
// Add magnetic coordinates
      exec($_SERVER["ODI_HOME"] . "/parsers/goes_bl");
      $pos = file("position.tmp", FILE_IGNORE_NEW_LINES);
      $vals = file("values.tmp", FILE_IGNORE_NEW_LINES);
      $tfile = fopen("load.tmp", "w");
      for ($i = 0; $i < count($pos); $i++)
        fprintf($tfile, "%s, %s%s", $pos[$i], $vals[$i], LF);
      fclose($tfile);

      @unlink("epoch.tmp");
      @unlink("position.tmp");
      @unlink("values.tmp");

      $result = TRUE;
    }
    else
      $result = FALSE;

    return $result;
  }

?>