<?php
/**
* Parser functions for the ODI database.
*
* When a new parser is written this file is edited to add a call to the new parse function.
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 2 December, 2009 
*
* @version $Id: parsefile.php 224 2009-12-03 12:49:21Z peter $
*
* @package ODI
* @subpackage Parsers
*/

/**
* Parse a data file and write formatted content to load.tmp.
*
* @param string   $rname The file name of the raw data file.
* @param int      $fid Raw data file id.
* @param string   $platform The platform name.
* @param string   $instrument The instrument name.
* @param string   $sk_file The name of the CDF settings file.
* @return bool True if succesful.
*/
function parseFile($rname, $fid, $platform, $instrument, $sk_file)
{
    $result = FALSE;
    
    // Generic parse function for cdf files
    if (strpos(strtolower($rname), ".cdf") !== FALSE)
    {
        $sname = substr_replace($sk_file, "set", -3, 3);
        if (strpos($rname, ".gz") !== FALSE)
        {
            @unlink("tmp.cdf");
            exec("gunzip -c " . $rname . " > tmp.cdf");
            $rname = "tmp.cdf";
        }
        exec($_SERVER["ODI_HOME"] . "/parsers/cdfexport -batch text -settings " . $sname . " -initial \"noheading,nooutput\" -text tmp.txt " . $rname);
        
        if (file_exists("tmp.txt"))
        {
            $tfile = fopen("load.tmp", "w");
            $ihandle = fopen("tmp.txt", "r");
            while (!feof($ihandle))
            {
                $sdum = fgets($ihandle);
                $sdum = preg_split("/\s+/", $sdum);
                if (count($sdum) > 2)
                {
                    $ts = array_shift($sdum) . " " . array_shift($sdum);
                    $dpos = strpos($ts, ".");
                    if ($dpos !== FALSE)
                    $ms = intval(substr($ts, $dpos+1));
                    else
                    $ms = 0;
                    $ts = strtotime($ts);
                    $epoch = date("Y-m-d H:i:s", $ts);
                    $cdf_epoch = date_to_cdfepoch($epoch, $ms);
                    fprintf($tfile, "%20.3f, %s, %d", $cdf_epoch, $epoch, $ms);
                    for ($i = 0; $i < count($sdum)-1; $i++)
                    fprintf($tfile, ",%s", $sdum[$i]);
                    fprintf($tfile, ", %d%s", $fid, LF);
                }
            }
            fclose($tfile);
            fclose($ihandle);
            @unlink("tmp.txt");
        
            $result = TRUE;
        }
        else
            $result = FALSE;
    }
    // Parse Dst.
    else if (strpos(strtolower($rname), "dst") !== FALSE)
    {
        if (function_exists("parse_Dst"))
        $result = parse_Dst($rname, $fid);
        else
            print "Function parse_Dst is not declared" . LF;
    }
    // Parse Kp and Ap.
    else if (($sk_file == "KpAp_3h.skt") || ($sk_file == "KpAp_1d.skt"))
    {
        if (function_exists("parse_KpAp"))
        $result = parse_KpAp($rname, $sk_file, $fid);
    else
        print "Function parse_KpAp is not declared" . LF;
    }
    // Parse GOES files
    else if ((substr($rname, -4) == ".TXT") &&
            ((substr($rname, -9, 1) == "5") || (substr($rname, -9, 1) == "1")) &&
            ((substr($rname, -12, 1) == "A") || (substr($rname, -12, 1) == "G") ||
            (substr($rname, -12, 1) == "H") || (substr($rname, -12, 1) == "I") ||
            (substr($rname, -12, 1) == "Z")))
    {
        if (function_exists("parse_GOES_SEM"))
            $result = parse_GOES_SEM($rname, $fid);
        else
            print "Function parse_GOES_SEM is not declared" . LF;
    }
    // Parse SSN_1m files
    else if ($sk_file == "SSN_1m.skt")
    {
        if (function_exists("parse_SSN_1m"))
            $result = parse_SSN_1m($rname, $fid);
        else
            print "Function parse_SSN_1m is not declared" . LF;
    }
    // Parse AZUR files
    else if ($sk_file == "EI-88.skt")
    {
        if (function_exists("parse_AZUR"))
            $result = parse_AZUR($rname, $fid);
        else
            print "Function parse_AZUR is not declared" . LF;
    }
    else
        print "No parse function has been defined for this dataset.";
    
    return $result;
}

?>