#!/usr/bin/php
<?php
/**
* Delete the data and metadata for a selected dataset.
*
* This program deletes all the data and metadata associated to a specific dataset.
* 
* Usage:
*
* <samp>
*  delete_dataset.php dataset_name
* </samp>
*
* where <var>dataset_name</var> is the SEDAT dataset name.
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 2 December, 2009 
*
* @version $Id: delete_dataset.php 224 2009-12-03 12:49:21Z peter $
*
* @package ODI
* @subpackage Tools
*/

require_once($_SERVER["ODI_HOME"].'/lib/odi.library.php');

// Get the input arguments.
$arg_array=array("sedat_name");
$args=get_cl_arguments($argv,$arg_array);

// Connect to the database at user level 1.
$conn_array=get_conn_array(1);
$conn=connect_to_database_array($conn_array);

// Check that the dataset exists and count the number of records, variables, ... that will be deleted.
$q = 'SELECT * FROM dataset WHERE sedat_name='.quote($args["sedat_name"]);
if (!($res=$conn->query($q))) printf("Query: %s\nError: %s\n",$q,$conn->error);
if($row=$res->fetch_assoc()) {
    fwrite(STDOUT,"===================\n".$args["sedat_name"]."\n------------\n");
    foreach($row as $key=>$value)
        fwrite(STDOUT,$key." = ".$value."\n");
    $dataset_id=$row["id"];
    $dataset_table=$row["name"];
    // Get number of global metadata.
    $q = 'SELECT COUNT(*) FROM dataset_metadata WHERE dataset_id='.$dataset_id;
    if (!($res=$conn->query($q))) printf("Query: %s\nError: %s\n",$q,$conn->error);
    $row=$res->fetch_array();
    fwrite(STDOUT,"Number of global metadata: ".$row[0]."\n");
    // Get number of variables.
    $q = 'SELECT COUNT(*) FROM variable_metadata WHERE dataset_id='.$dataset_id;
    if (!($res=$conn->query($q))) printf("Query: %s\nError: %s\n",$q,$conn->error);
    $row=$res->fetch_array();
    fwrite(STDOUT,"Number of variables: ".$row[0]."\n");
    // Get number of records.
    $q = 'SELECT COUNT(*) FROM '.$dataset_table;
    if (!($res=$conn->query($q))) printf("Query: %s\nError: %s\n",$q,$conn->error);
    $row=$res->fetch_array();
    fwrite(STDOUT,"Number of records: ".$row[0]."\n");
    // Query if to delete.
    fwrite(STDOUT," Delete? (y/[n]): ");
    $input=trim(fgets(STDIN));
    if($input=='y') {
        fwrite(STDOUT,"Deleting ".$args["sedat_name"]." ... ");
        $q = 'DROP TABLE '.$dataset_table;
        if (!($res=$conn->query($q))) printf("Query: %s\nError: %s\n",$q,$conn->error);
        $q = 'DELETE FROM dataset WHERE id='.$dataset_id;
        if (!($res=$conn->query($q))) printf("Query: %s\nError: %s\n",$q,$conn->error);
        fwrite(STDOUT,"Done!\n");
    }
    else {
        fwrite(STDOUT,"Will not delete!\n");
    }
}
else {
    fwrite(STDOUT,$args["sedat_name"]." does not exist!\n");
    exit(0);
}
?>
