#!/usr/bin/php
<?php
/**
* Get the SEDAT name, ODI name, number of records and epoch range for the datasets.
*
* Usage:
*
* <samp>
* show_datasets.php [dataset]
* </samp>
*
* where <samp>dataset</samp> is the SEDAT name of the dataset. If <samp>dataset</samp> is not given
* then all datasets are listed.
* 
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0 
*
* Release Date: 2 December, 2009 
*
* @version $Id: show_datasets.php 224 2009-12-03 12:49:21Z peter $
*
* @package ODI
* @subpackage Tools
*/

require_once($_SERVER["ODI_HOME"].'/lib/odi.library.php');

// Get the input arguments.
$dataset="%";
if(count($argv)>1) {
    $dataset=$argv[1];
}
elseif(count($argv)>2) {
    fwrite(STDOUT,"Error: To many input arguments.\n");
    fwrite(STDOUT,"Usage: show_datasets.php [<dataset>].\n");
}

// Connect to the database at user level 3.
$conn_array=get_conn_array(3);
$conn=connect_to_database_array($conn_array);

// Get the epoch range.
//$q = 'SELECT MIN(epoch) AS t1, MAX(epoch) AS t2 FROM '.no_quote($args["dataset"]);
$q = 'SELECT name,sedat_name,epoch_start,epoch_end FROM dataset WHERE sedat_name LIKE '.quote($dataset);
if (!($res=$conn->query($q))) printf("Query: %s\nError: %s\n",$q,$conn->error);
while($row=$res->fetch_assoc()) {
    fwrite(STDOUT,$row["sedat_name"]." (".$row["name"].") ");
    $q = 'SELECT COUNT(cdf_epoch) AS num_rec FROM '.$row["name"];
    if (!($res_count=$conn->query($q))) printf("Query: %s\nError: %s\n",$q,$conn->error);
    $count = $res_count->fetch_assoc();
    fwrite(STDOUT,$count["num_rec"]." records: ");
    if($row["epoch_start"]) {
        list($date,$ms) = cdfepoch_to_date($row["epoch_start"]);
        fwrite(STDOUT,$date.".".str_pad($ms,3,"0",STR_PAD_LEFT)." to ");
        list($date,$ms) = cdfepoch_to_date($row["epoch_end"]);
        fwrite(STDOUT,$date.".".str_pad($ms,3,"0",STR_PAD_LEFT)."\n");
    }
    else {
        fwrite(STDOUT,"No data!\n");
    }
}

?>