#!/usr/bin/php
<?php
/**
* Show the metadata for a selected dataset.
*
* Usage:
*
* <samp>
*  show_metadata.php dataset_name
* </samp>
*
* where <samp>dataset_name</samp> is the SEDAT dataset name.
*
* <pre>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* <>          ___  ___  __ _                                        <>
* <>         / _ \/ __|/ _` |               european space agency   <>
* <>        |  __/\__ \ (_| |                                       <>
* <>         \___/\___/\__,_|            agence spatiale europeenne <>
* <>                                                                <>
* <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
* </pre>
* The IPR of this software belongs to the European Space Agency. 
* The usage is at the discretion of the Agency and licenses for its 
* use should be addressed to the Contracts Dept. at ESTEC: 
* <pre>
* European Space Agency ESA/ESTEC 
* Contracts Department 
* Postbus 299 
* 2200 AG Noorwijk 
* The Netherlands 
* 
* WEB:         http://www.esa.int/ 
* Phone:         +31 71 565-6565 
* </pre>
* The software was developed under the ESA/ESTEC contract 21964/08/NL/AT by: 
* <pre>
* Peter Wintoft, Swedish Institute of Space Physics and 
* Daniel Heynderickx, DH Consultancy
* </pre>
* 
* Version number: 1.0
* 
* Release Date: 2 December, 2009 
*
* @version $Id: show_metadata.php 224 2009-12-03 12:49:21Z peter $
*
* @package ODI
* @subpackage Tools
*/

require_once($_SERVER["ODI_HOME"].'/lib/odi.library.php');

// Get the input arguments.
$arg_array=array("dataset");
$args=get_cl_arguments($argv,$arg_array);

// Connect to the database at user level 3.
$conn_array=get_conn_array(3);
$conn=connect_to_database_array($conn_array);

// Get the "dataset" metadata.
$q = 'SELECT * FROM dataset WHERE sedat_name='.quote($args["dataset"]);
if (!($res=$conn->query($q))) printf("Query: %s\nError: %s\n",$q,$conn->error);
if($row=$res->fetch_assoc()) {
    fwrite(STDOUT,"===================\n".$args["dataset"]."\n------------\n");
    foreach($row as $key=>$value)
        fwrite(STDOUT,$key." = ".$value."\n");
}
else {
    fwrite(STDOUT,$args["dataset"]." does not exist!\n");
    exit(0);
}

// Get the "dataset_metadata".
$q = 'SELECT attribute, cdf_type, entry_number, value';
$q.= ' FROM dataset_metadata';
$q.= ' LEFT JOIN dataset ON dataset_metadata.dataset_id=dataset.id';
$q.= ' LEFT JOIN data_type ON dataset_metadata.data_type_id=data_type.id';
$q.= ' WHERE dataset.sedat_name='.quote($args["dataset"]);
if (!($res=$conn->query($q))) printf("Query: %s\nError: %s\n",$q,$conn->error);
fwrite(STDOUT,"----------------\nGlobal attributes\n----------------\n");
while($row=$res->fetch_assoc()) {
    foreach($row as $key=>$value) {
        fwrite(STDOUT,$key." = ".$value."\n");
    }
    fwrite(STDOUT,"----------------\n");
}

// Get the "variable_metadata".
$q = 'SELECT data_type.cdf_type AS data_type,';
$q .= ' variable_metadata.name AS name';
$q .= ' FROM dataset JOIN variable_metadata JOIN data_type';
$q .= ' ON dataset.sedat_name='.quote($args["dataset"]);
$q .= ' AND variable_metadata.dataset_id = dataset.id';
$q .= ' AND variable_metadata.data_type_id = data_type.id';
$q .= ' ORDER BY name';
if (!($res=$conn->query($q))) printf("Query: %s\nError: %s\n",$q,$conn->error);
fwrite(STDOUT,"----------------\nVariables\n----------------\n");
while($row=$res->fetch_assoc()) {
    fwrite(STDOUT,$row["name"]." ".$row["data_type"]."\n");
}

// Get the "variable_attribute".
$q = 'SELECT variable_metadata.name AS var_name';
$q .= ',variable_attribute.name AS att_name';
$q .= ',variable_attribute.value AS value';
$q .= ' FROM dataset JOIN variable_metadata JOIN variable_attribute';
$q .= ' ON dataset.sedat_name='.quote($args["dataset"]);
$q .= ' AND variable_metadata.dataset_id = dataset.id';
$q .= ' AND variable_attribute.variable_metadata_id = variable_metadata.id';
$q .= ' ORDER BY var_name,att_name';
if (!($res=$conn->query($q))) printf("Query: %s\nError: %s\n",$q,$conn->error);
fwrite(STDOUT,"----------------\nVariable attributes\n----------------\n");
while($row=$res->fetch_assoc()) {
    fwrite(STDOUT,$row["var_name"]." ".$row["att_name"]." ".$row["value"]."\n");
}
?>